@@ -97,23 +97,23 @@ static struct vu_fs_discipline url_server_socket_vtable;
 
 
 /* __STDC__ prototypes for static functions */
-static regex_t * url_socket_client_regex (t_uchar ** name,
-                                          t_uchar *** doc,
+static regex_t * url_socket_client_regex (const t_uchar ** name,
+                                          const t_uchar *** doc,
                                           enum url_socket_domains domains,
                                           int default_port);
-static regex_t * url_socket_server_regex (t_uchar ** name,
-                                          t_uchar *** doc,
+static regex_t * url_socket_server_regex (const t_uchar ** name,
+                                          const t_uchar *** doc,
                                           enum url_socket_domains domains,
                                           int may_be_client,
                                           int default_port);
-static regex_t * url_socket_only_server_regex (t_uchar ** name,
-                                               t_uchar *** doc,
+static regex_t * url_socket_only_server_regex (const t_uchar ** name,
+                                               const t_uchar *** doc,
                                                enum url_socket_domains domains,
                                                int default_port);
 static int server_fd_for_addr (int * errn,
                                struct sockaddr * addr,
                                int addr_len,
-                               char * path,
+                               const char * path,
                                int flags,
                                int mode,
                                struct url_socket_params * params);
@@ -124,17 +124,17 @@ static int decode_socket_url (int * errn,
                               int * addr_len,
                               struct url_socket_params * params,
                               int * be_a_server,
-                              char * path,
+                              const char * path,
                               int be_a_server_hint);
 static void * url_socket_make_closure (void * closure);
 static void url_socket_free_closure (void * closure);
-static int url_socket_open (int * errn, char * path, int flags, int mode, void * closure);
+static int url_socket_open (int * errn, const char * path, int flags, int mode, void * closure);
 static int url_socket_close (int * errn, int fd, void * closure);
-static int url_socket_access (int * errn, char * path, int mode, void * closure);
-static int url_socket_chdir (int * errn, char * path, void * closure);
-static int url_socket_chmod (int * errn, char * path, int mode, void * closure);
-static int url_socket_chown (int * errn, char * path, int owner, int group, void * closure);
-static int url_socket_chroot (int * errn, char * path, void * closure);
+static int url_socket_access (int * errn, const char * path, int mode, void * closure);
+static int url_socket_chdir (int * errn, const char * path, void * closure);
+static int url_socket_chmod (int * errn, const char * path, int mode, void * closure);
+static int url_socket_chown (int * errn, const char * path, int owner, int group, void * closure);
+static int url_socket_chroot (int * errn, const char * path, void * closure);
 static int url_socket_closedir (int * errn, DIR * dir, void * closure);
 static int url_socket_fchdir (int * errn, int fd, void * closure);
 static int url_socket_fchmod (int * errn, int fd, int mode, void * closure);
@@ -142,22 +142,22 @@ static int url_socket_fchown (int * errn, int fd, int owner, int group, void * c
 static int url_socket_fstat (int * errn, int fd, struct stat * buf, void * closure);
 static int url_socket_fsync (int * errn, int fd, void * closure);
 static int url_socket_ftruncate (int * errn, int fd, off_t where, void * closure);
-static int url_socket_link (int * errn, char * from, char * to, void * closure);
+static int url_socket_link (int * errn, const char * from, const char * to, void * closure);
 static off_t url_socket_lseek (int * errn, int fd, off_t offset, int whence, void * closure);
-static int url_socket_lstat (int * errn, char * path, struct stat * buf, void * closure);
-static int url_socket_mkdir (int * errn, char * path, int mode, void * closure);
-static int url_socket_opendir (int * errn, DIR ** retv,  char * path, void * closure);
+static int url_socket_lstat (int * errn, const char * path, struct stat * buf, void * closure);
+static int url_socket_mkdir (int * errn, const char * path, int mode, void * closure);
+static int url_socket_opendir (int * errn, DIR ** retv,  const char * path, void * closure);
 static ssize_t url_socket_read (int * errn, int fd, char * buf, size_t count, void * closure);
 static int url_socket_readdir (int * errn, struct alloc_limits * limits, char ** file_ret, DIR * dir, void * closure);
-static int url_socket_readlink (int * errn, char * path, char * buf, int bufsize, void * closure);
-static int url_socket_rename (int * errn, char * from, char * to, void * closure);
-static int url_socket_rmdir (int * errn, char * path, void * closure);
-static int url_socket_stat (int * errn, char * path, struct stat * buf, void * closure);
-static int url_socket_symlink (int * errn, char * from, char * to, void * closure);
-static int url_socket_truncate (int * errn, char * path, off_t where, void * closure);
-static int url_socket_unlink (int * errn, char * path, void * closure);
-static int url_socket_utime (int * errn, char * path, struct utimbuf * times, void * closure);
-static ssize_t url_socket_write (int * errn, int fd, char * buf, size_t count, void * closure);
+static int url_socket_readlink (int * errn, const char * path, char * buf, int bufsize, void * closure);
+static int url_socket_rename (int * errn, const char * from, const char * to, void * closure);
+static int url_socket_rmdir (int * errn, const char * path, void * closure);
+static int url_socket_stat (int * errn, const char * path, struct stat * buf, void * closure);
+static int url_socket_symlink (int * errn, const char * from, const char * to, void * closure);
+static int url_socket_truncate (int * errn, const char * path, off_t where, void * closure);
+static int url_socket_unlink (int * errn, const char * path, void * closure);
+static int url_socket_utime (int * errn, const char * path, struct utimbuf * times, void * closure);
+static ssize_t url_socket_write (int * errn, int fd, const char * buf, size_t count, void * closure);
 static int url_socket_fcntl (int * errn, int fd, int cmd, long arg, void * closure);
 static int url_socket_dup (int * errn, int fd, void * closure);
 static int url_socket_dup2 (int * errn, int fd, int newfd, void * closure);
@@ -168,8 +168,8 @@ static int url_server_socket_close (int * errn, int fd, void * closure);
 
 
 
-/* static regex_t * url_socket_client_regex (t_uchar * name,
- *                                           t_uchar *** doc,
+/* static regex_t * url_socket_client_regex (const t_uchar * name,
+ *                                           const t_uchar *** doc,
  *                                           enum url_socket_domains domains,
  *                                           int default_port);
  *
@@ -183,8 +183,8 @@ static int url_server_socket_close (int * errn, int fd, void * closure);
  *      both                    unix:\|inet://          client/any
  */
 static regex_t *
-url_socket_client_regex (t_uchar ** name,
-                         t_uchar *** doc,
+url_socket_client_regex (const t_uchar ** name,
+                         const t_uchar *** doc,
                          enum url_socket_domains domains,
                          int default_port)
 {
@@ -196,7 +196,7 @@ url_socket_client_regex (t_uchar ** name,
       {
         static int initialized = 0;
         static regex_t regex;
-        static t_uchar * docstr[] =
+        static const t_uchar * docstr[] =
           {
             "unix:$PATH",
             "Client connection to a unix-domain socket.",
@@ -219,14 +219,14 @@ url_socket_client_regex (t_uchar ** name,
       {
         static int initialized = 0;
         static regex_t regex;
-        static t_uchar * docstr[] =
+        static const t_uchar * docstr[] =
           {
             "inet:$HOST:$PORT",
             "Client connection to an internet-domain",
             "socket.",
             0
           };
-        static t_uchar * docstr_opt_port[] =
+        static const t_uchar * docstr_opt_port[] =
           {
             "inet:$HOST[:$PORT]",
             "Client connection to an internet-domain",
@@ -248,7 +248,7 @@ url_socket_client_regex (t_uchar ** name,
       {
         static int initialized = 0;
         static regex_t regex;
-        static t_uchar * docstr[] =
+        static const t_uchar * docstr[] =
           {
             "unix:$PATH or inet:$HOST:$PORT",
             "Client connection to a unix or",
@@ -256,7 +256,7 @@ url_socket_client_regex (t_uchar ** name,
             "is ~-expanded.",
             0
           };
-        static t_uchar * docstr_opt_port[] =
+        static const t_uchar * docstr_opt_port[] =
           {
             "unix:$PATH or inet:$HOST[:$PORT]",
             "Client connection to a unix or",
@@ -279,8 +279,8 @@ url_socket_client_regex (t_uchar ** name,
 }
 
 
-/* static regex_t * url_socket_server_regex (t_uchar ** name,
- *                                           t_uchar *** doc,
+/* static regex_t * url_socket_server_regex (const t_uchar ** name,
+ *                                           const t_uchar *** doc,
  *                                           enum url_socket_domains domains,
  *                                           int may_be_client,
  *                                           int default_port);
@@ -299,8 +299,8 @@ url_socket_client_regex (t_uchar ** name,
  * associated with each combination of `domains' and `may_be_client'.
  */
 static regex_t *
-url_socket_server_regex (t_uchar ** name,
-                         t_uchar *** doc,
+url_socket_server_regex (const t_uchar ** name,
+                         const t_uchar *** doc,
                          enum url_socket_domains domains,
                          int may_be_client,
                          int default_port)
@@ -322,7 +322,7 @@ url_socket_server_regex (t_uchar ** name,
           }
         if (may_be_client)
           {
-            static t_uchar * docstr[] =
+            static const t_uchar * docstr[] =
               {
                 "unix:$PATH or unix-server:$PATH",
                 "Be a server or client for a unix-domain",
@@ -334,7 +334,7 @@ url_socket_server_regex (t_uchar ** name,
           }
         else
           {
-            static t_uchar * docstr[] =
+            static const t_uchar * docstr[] =
               {
                 "unix:$PATH or unix-server:$PATH",
                 "Be a server for a unix-domain socket.",
@@ -359,7 +359,7 @@ url_socket_server_regex (t_uchar ** name,
           }
         if (may_be_client)
           {
-            static t_uchar * docstr[] =
+            static const t_uchar * docstr[] =
               {
                 "inet:$HOST:$PORT",
                 "  or inet-server:$PATH:$PORT",
@@ -367,7 +367,7 @@ url_socket_server_regex (t_uchar ** name,
                 "internet-domain socket.",
                 0
               };
-            static t_uchar * docstr_opt_port[] =
+            static const t_uchar * docstr_opt_port[] =
               {
                 "inet:$HOST[:$PORT]",
                 "  or inet-server:$PATH[:$PORT]",
@@ -380,7 +380,7 @@ url_socket_server_regex (t_uchar ** name,
           }
         else
           {
-            static t_uchar * docstr[] =
+            static const t_uchar * docstr[] =
               {
                 "inet:$HOST:$PORT",
                 "  or inet-server:$PATH:$PORT",
@@ -388,7 +388,7 @@ url_socket_server_regex (t_uchar ** name,
                 "socket.",
                 0
               };
-            static t_uchar * docstr_opt_port[] =
+            static const t_uchar * docstr_opt_port[] =
               {
                 "inet:$HOST[:$PORT]",
                 "  or inet-server:$PATH[:$PORT]",
@@ -413,7 +413,7 @@ url_socket_server_regex (t_uchar ** name,
           }
         if (may_be_client)
           {
-            static t_uchar * docstr[] =
+            static const t_uchar * docstr[] =
               {
                 "unix:$PATH, unix-server:$PATH",
                 "inet:$HOST:$PORT",
@@ -423,7 +423,7 @@ url_socket_server_regex (t_uchar ** name,
                 "~-expanded.",
                 0
               };
-            static t_uchar * docstr_opt_port[] =
+            static const t_uchar * docstr_opt_port[] =
               {
                 "unix:$PATH, unix-server:$PATH",
                 "inet:$HOST:[$PORT]",
@@ -437,7 +437,7 @@ url_socket_server_regex (t_uchar ** name,
           }
         else
           {
-            static t_uchar * docstr[] =
+            static const t_uchar * docstr[] =
               {
                 "unix:$PATH, unix-server:$PATH",
                 "inet:$HOST:$PORT",
@@ -447,7 +447,7 @@ url_socket_server_regex (t_uchar ** name,
                 "~-expanded.",
                 0
               };
-            static t_uchar * docstr_opt_port[] =
+            static const t_uchar * docstr_opt_port[] =
               {
                 "unix:$PATH, unix-server:$PATH",
                 "inet:$HOST:[$PORT]",
@@ -465,8 +465,8 @@ url_socket_server_regex (t_uchar ** name,
 }
 
 
-/* static regex_t * url_socket_only_server_regex (t_uchar ** name,
- *                                                t_uchar *** doc,
+/* static regex_t * url_socket_only_server_regex (const t_uchar ** name,
+ *                                                const t_uchar *** doc,
  *                                                enum url_socket_domains domains,
  *                                                int default_port);
  *
@@ -484,8 +484,8 @@ url_socket_server_regex (t_uchar ** name,
  * associated with each domain.
  */
 static regex_t *
-url_socket_only_server_regex (t_uchar ** name,
-                              t_uchar *** doc,
+url_socket_only_server_regex (const t_uchar ** name,
+                              const t_uchar *** doc,
                               enum url_socket_domains domains,
                               int default_port)
 {
@@ -497,7 +497,7 @@ url_socket_only_server_regex (t_uchar ** name,
       {
         static int initialized = 0;
         static regex_t regex;
-        static t_uchar * docstr[] =
+        static const t_uchar * docstr[] =
           {
             "unix-server:$PATH",
             "Be a server for a unix-domain socket.",
@@ -519,14 +519,14 @@ url_socket_only_server_regex (t_uchar ** name,
       {
         static int initialized = 0;
         static regex_t regex;
-        static t_uchar * docstr[] =
+        static const t_uchar * docstr[] =
           {
             "inet-server:$PATH:$PORT",
             "Be a server for an internet-domain",
             "socket.",
             0
           };
-        static t_uchar * docstr_opt_port[] =
+        static const t_uchar * docstr_opt_port[] =
           {
             "inet-server:$PATH[:$PORT]",
             "Be a server for an internet-domain",
@@ -548,7 +548,7 @@ url_socket_only_server_regex (t_uchar ** name,
       {
         static int initialized = 0;
         static regex_t regex;
-        static t_uchar * docstr[] =
+        static const t_uchar * docstr[] =
           {
             "unix-server:$PATH",
             "or inet-server:$HOST:$PORT",
@@ -557,7 +557,7 @@ url_socket_only_server_regex (t_uchar ** name,
             "~-expanded.",
             0
           };
-        static t_uchar * docstr_opt_port[] =
+        static const t_uchar * docstr_opt_port[] =
           {
             "unix-server:$PATH",
             "or inet-server:$HOST:[$PORT]",
@@ -622,8 +622,8 @@ url_socket_push_client_handler (enum url_socket_domains domains,
 {
   regex_t * regex;
   struct url_socket_params params;
-  t_uchar * name;
-  t_uchar ** doc;
+  const t_uchar * name;
+  const t_uchar ** doc;
 
   regex = url_socket_client_regex (&name, &doc, domains, default_port);
   params.type = url_socket_client;
@@ -811,8 +811,8 @@ url_socket_push_server_handler (enum url_socket_domains domains,
 {
   regex_t * regex;
   struct url_socket_params params;
-  t_uchar * name;
-  t_uchar ** doc;
+  const t_uchar * name;
+  const t_uchar ** doc;
 
   if (only_server_url)
     regex = url_socket_only_server_regex (&name, &doc, domains, default_port);
@@ -875,7 +875,7 @@ static union
 /* static int server_fd_for_addr (int * errn,
  *		    		  struct sockaddr * addr,
  *		    		  int addr_len,
- *				  char * path,
+ *				  const char * path,
  *		    		  int flags,
  *		    		  int mode,
  *		    		  struct url_socket_params * params);
@@ -899,7 +899,7 @@ static int
 server_fd_for_addr (int * errn,
 		    struct sockaddr * addr,
 		    int addr_len,
-		    char * path,
+		    const char * path,
 		    int flags,
 		    int mode,
 		    struct url_socket_params * params)
@@ -1064,7 +1064,7 @@ remove_memo_for_server_fd (int fd)
  *		   		 struct sockaddr * addr,
  *		   		 int * addr_len,
  *		   		 int * be_a_server,
- *				 char * path,
+ *				 const char * path,
  *		   		 struct url_socket_params * params,
  *		   		 int be_a_server_hint);
  *
@@ -1098,7 +1098,7 @@ decode_socket_url (int * errn,
 		   int * addr_len,
 		   struct url_socket_params * params,
 		   int * be_a_server,
-		   char * path,
+		   const char * path,
 		   int be_a_server_hint)
 {
   int is_server_address;
@@ -1571,7 +1571,7 @@ url_socket_free_closure (void * closure)
 
 
 /*c void url_socket_open (int * errn,
- * 			  char * path,
+ * 			  const char * path,
  *			  int flags,
  *			  int mode,
  *			  void * closure);
@@ -1581,7 +1581,7 @@ url_socket_free_closure (void * closure)
  * See `url_socket_push_client_handler' and `url_socket_push_server_handler'.
  */
 static int
-url_socket_open (int * errn, char * path, int flags, int mode, void * closure)
+url_socket_open (int * errn, const char * path, int flags, int mode, void * closure)
 {
   struct url_socket_params * params;
   char addr_space[SOCK_MAXADDRLEN];
@@ -1679,7 +1679,7 @@ url_socket_close (int * errn, int fd, void * closure)
  */
 
 static int
-url_socket_access (int * errn, char * path, int mode, void * closure)
+url_socket_access (int * errn, const char * path, int mode, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1687,7 +1687,7 @@ url_socket_access (int * errn, char * path, int mode, void * closure)
 
 
 static int
-url_socket_chdir (int * errn, char * path, void * closure)
+url_socket_chdir (int * errn, const char * path, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1695,7 +1695,7 @@ url_socket_chdir (int * errn, char * path, void * closure)
 
 
 static int
-url_socket_chmod (int * errn, char * path, int mode, void * closure)
+url_socket_chmod (int * errn, const char * path, int mode, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1703,7 +1703,7 @@ url_socket_chmod (int * errn, char * path, int mode, void * closure)
 
 
 static int
-url_socket_chown (int * errn, char * path, int owner, int group, void * closure)
+url_socket_chown (int * errn, const char * path, int owner, int group, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1711,7 +1711,7 @@ url_socket_chown (int * errn, char * path, int owner, int group, void * closure)
 
 
 static int
-url_socket_chroot (int * errn, char * path, void * closure)
+url_socket_chroot (int * errn, const char * path, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1775,7 +1775,7 @@ url_socket_ftruncate (int * errn, int fd, off_t where, void * closure)
 
 
 static int
-url_socket_link (int * errn, char * from, char * to, void * closure)
+url_socket_link (int * errn, const char * from, const char * to, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1791,7 +1791,7 @@ url_socket_lseek (int * errn, int fd, off_t offset, int whence, void * closure)
 
 
 static int
-url_socket_lstat (int * errn, char * path, struct stat * buf, void * closure)
+url_socket_lstat (int * errn, const char * path, struct stat * buf, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1799,7 +1799,7 @@ url_socket_lstat (int * errn, char * path, struct stat * buf, void * closure)
 
 
 static int
-url_socket_mkdir (int * errn, char * path, int mode, void * closure)
+url_socket_mkdir (int * errn, const char * path, int mode, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1807,7 +1807,7 @@ url_socket_mkdir (int * errn, char * path, int mode, void * closure)
 
 
 static int
-url_socket_opendir (int * errn, DIR ** retv,  char * path, void * closure)
+url_socket_opendir (int * errn, DIR ** retv,  const char * path, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1830,7 +1830,7 @@ url_socket_readdir (int * errn, struct alloc_limits * limits, char ** file_ret,
 
 
 static int
-url_socket_readlink (int * errn, char * path, char * buf, int bufsize, void * closure)
+url_socket_readlink (int * errn, const char * path, char * buf, int bufsize, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1838,7 +1838,7 @@ url_socket_readlink (int * errn, char * path, char * buf, int bufsize, void * cl
 
 
 static int
-url_socket_rename (int * errn, char * from, char * to, void * closure)
+url_socket_rename (int * errn, const char * from, const char * to, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1846,7 +1846,7 @@ url_socket_rename (int * errn, char * from, char * to, void * closure)
 
 
 static int
-url_socket_rmdir (int * errn, char * path, void * closure)
+url_socket_rmdir (int * errn, const char * path, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1854,7 +1854,7 @@ url_socket_rmdir (int * errn, char * path, void * closure)
 
 
 static int
-url_socket_stat (int * errn, char * path, struct stat * buf, void * closure)
+url_socket_stat (int * errn, const char * path, struct stat * buf, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1862,7 +1862,7 @@ url_socket_stat (int * errn, char * path, struct stat * buf, void * closure)
 
 
 static int
-url_socket_symlink (int * errn, char * from, char * to, void * closure)
+url_socket_symlink (int * errn, const char * from, const char * to, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1870,7 +1870,7 @@ url_socket_symlink (int * errn, char * from, char * to, void * closure)
 
 
 static int
-url_socket_truncate (int * errn, char * path, off_t where, void * closure)
+url_socket_truncate (int * errn, const char * path, off_t where, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1878,7 +1878,7 @@ url_socket_truncate (int * errn, char * path, off_t where, void * closure)
 
 
 static int
-url_socket_unlink (int * errn, char * path, void * closure)
+url_socket_unlink (int * errn, const char * path, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1886,7 +1886,7 @@ url_socket_unlink (int * errn, char * path, void * closure)
 
 
 static int
-url_socket_utime (int * errn, char * path, struct utimbuf * times, void * closure)
+url_socket_utime (int * errn, const char * path, struct utimbuf * times, void * closure)
 {
   *errn = EINVAL;
   return -1;
@@ -1894,7 +1894,7 @@ url_socket_utime (int * errn, char * path, struct utimbuf * times, void * closur
 
 
 static ssize_t
-url_socket_write (int * errn, int fd, char * buf, size_t count, void * closure)
+url_socket_write (int * errn, int fd, const char * buf, size_t count, void * closure)
 {
   return vu_sys_write (errn, fd, buf, count, 0);
 }
