@@ -28,10 +28,10 @@
 /* __STDC__ prototypes for static functions */
 static void mirror_revision (struct arch_archive * arch,
                              struct arch_archive * to_arch,
-                             t_uchar * revision,
-                             t_uchar * prev_level,
-			     struct arch_archive_mirror_options * archive_mirror_opts);
-static int list_contains (rel_table list, t_uchar * elt);
+                             const t_uchar * revision,
+                             const t_uchar * prev_level,
+                             struct arch_archive_mirror_options * archive_mirror_opts);
+static int list_contains (rel_table list, const t_uchar * elt);
 
 
 
@@ -46,9 +46,9 @@ arch_archive_mirror (int chatter_fd,
   t_uchar * only_branch = 0;
   t_uchar * only_version = 0;
   t_uchar * only_revision = 0;
-  rel_table categories = 0;
-  rel_table to_categories = 0;
-  rel_table missing_categories = 0;
+  rel_table categories = rel_table_nil;
+  rel_table to_categories = rel_table_nil;
+  rel_table missing_categories = rel_table_nil;
   int c;
   t_uchar * errstr = 0;
 
@@ -86,92 +86,92 @@ arch_archive_mirror (int chatter_fd,
 
   for (c = 0; c < rel_n_records (categories); ++c)
     {
-      rel_table branches = 0;
-      rel_table to_branches = 0;
-      rel_table missing_branches = 0;
+      rel_table branches = rel_table_nil;
+      rel_table to_branches = rel_table_nil;
+      rel_table missing_branches = rel_table_nil;
       int b;
 
-      if (only_category && str_cmp (only_category, categories[c][0]))
+      if (only_category && str_cmp (only_category, rel_peek_str (categories, c, 0)))
         continue;
 
-      if (list_contains (missing_categories, categories[c][0]))
+      if (list_contains (missing_categories, rel_peek_str (categories, c, 0)))
         {
-          arch_chatter (chatter_fd, "** adding category %s\n", categories[c][0]);
-          if (arch_make_category (&errstr, to_arch, categories[c][0]))
+          arch_chatter (chatter_fd, "** adding category %s\n", rel_peek_str (categories, c, 0));
+          if (arch_make_category (&errstr, to_arch, rel_peek_str (categories, c, 0)))
             {
-              safe_printfmt (2, "archive-mirror: error creating category %s (%s)\n", categories[c][0], errstr);
+              safe_printfmt (2, "archive-mirror: error creating category %s (%s)\n", rel_peek_str (categories, c, 0), errstr);
               exit (2);
             }
         }
 
-      branches = arch_archive_branches (arch, categories[c][0]);
-      to_branches = arch_archive_branches (to_arch, categories[c][0]);
+      branches = arch_archive_branches (arch, rel_peek_str (categories, c, 0));
+      to_branches = arch_archive_branches (to_arch, rel_peek_str (categories, c, 0));
       missing_branches = rel_join (1, rel_join_output (1,0, -1), 0, 0, branches, to_branches);
 
       for (b = 0; b < rel_n_records (branches); ++b)
         {
-          rel_table versions = 0;
-          rel_table to_versions = 0;
-          rel_table missing_versions = 0;
+          rel_table versions = rel_table_nil;
+          rel_table to_versions = rel_table_nil;
+          rel_table missing_versions = rel_table_nil;
           int v;
 
-          if (only_branch && str_cmp (only_branch, branches[b][0]))
+          if (only_branch && str_cmp (only_branch, rel_peek_str (branches, b, 0)))
             continue;
 
-          if (list_contains (missing_branches, branches[b][0]))
+          if (list_contains (missing_branches, rel_peek_str (branches, b, 0)))
             {
-              arch_chatter (chatter_fd, "** adding branch %s\n", branches[b][0]);
-              if (arch_make_branch (&errstr, to_arch, branches[b][0]))
+              arch_chatter (chatter_fd, "** adding branch %s\n", rel_peek_str (branches, b, 0));
+              if (arch_make_branch (&errstr, to_arch, rel_peek_str (branches, b, 0)))
                 {
-                  safe_printfmt (2, "archive-mirror: error creating branch %s (%s)\n", branches[b][0], errstr);
+                  safe_printfmt (2, "archive-mirror: error creating branch %s (%s)\n", rel_peek_str (branches, b, 0), errstr);
                   exit (2);
                 }
             }
 
-          versions = arch_archive_versions (arch, branches[b][0]);
-          to_versions = arch_archive_versions (to_arch, branches[b][0]);
+          versions = arch_archive_versions (arch, rel_peek_str (branches, b, 0));
+          to_versions = arch_archive_versions (to_arch, rel_peek_str (branches, b, 0));
           missing_versions = rel_join (1, rel_join_output (1,0, -1), 0, 0, versions, to_versions);
 
           for (v = 0; v < rel_n_records (versions); ++v)
             {
-              rel_table revisions = 0;
-              rel_table to_revisions = 0;
-              rel_table missing_revisions = 0;
+              rel_table revisions = rel_table_nil;
+              rel_table to_revisions = rel_table_nil;
+              rel_table missing_revisions = rel_table_nil;
               int r;
 
-              if (only_version && str_cmp (only_version, versions[v][0]))
+              if (only_version && str_cmp (only_version, rel_peek_str (versions, v, 0)))
                 continue;
 
-              if (list_contains (missing_versions, versions[v][0]))
+              if (list_contains (missing_versions, rel_peek_str (versions, v, 0)))
                 {
-                  arch_chatter (chatter_fd, "** adding version %s\n", versions[v][0]);
-                  if (arch_make_version (&errstr, to_arch, versions[v][0]))
+                  arch_chatter (chatter_fd, "** adding version %s\n", rel_peek_str (versions, v, 0));
+                  if (arch_make_version (&errstr, to_arch, rel_peek_str (versions, v, 0)))
                     {
-                      safe_printfmt (2, "archive-mirror: error creating version %s (%s)\n", versions[v][0], errstr);
+                      safe_printfmt (2, "archive-mirror: error creating version %s (%s)\n", rel_peek_str (versions, v, 0), errstr);
                       exit (2);
                     }
                 }
 
-              revisions = arch_archive_revisions (arch, versions[v][0], 2);
-              to_revisions = arch_archive_revisions (to_arch, versions[v][0], 2);
+              revisions = arch_archive_revisions (arch, rel_peek_str (versions, v, 0), 2);
+              to_revisions = arch_archive_revisions (to_arch, rel_peek_str (versions, v, 0), 2);
               missing_revisions = rel_join (1, rel_join_output (1,0, -1), 0, 0, revisions, to_revisions);
 
               for (r = 0; r < rel_n_records (revisions); ++r)
                 {
-                  if (only_revision && str_cmp (only_revision, revisions[r][0]))
+                  if (only_revision && str_cmp (only_revision, rel_peek_str (revisions, r, 0)))
                     continue;
 
-                  if (list_contains (missing_revisions, revisions[r][0]))
+                  if (list_contains (missing_revisions, rel_peek_str (revisions, r, 0)))
                     {
                       t_uchar * prev_level = 0;
 
                       if (r)
                         {
-                          prev_level = arch_parse_package_name (arch_ret_patch_level, 0, revisions[r - 1][0]);
+                          prev_level = arch_parse_package_name (arch_ret_patch_level, 0, rel_peek_str (revisions, r - 1, 0));
                         }
 
-                      arch_chatter (chatter_fd, "** adding revision %s\n", revisions[r][0]);
-                      mirror_revision (arch, to_arch, revisions[r][0], prev_level, archive_mirror_opts);
+                      arch_chatter (chatter_fd, "** adding revision %s\n", rel_peek_str (revisions, r, 0));
+                      mirror_revision (arch, to_arch, rel_peek_str (revisions, r, 0), prev_level, archive_mirror_opts);
 
                       lim_free (0, prev_level);
                     }
@@ -208,8 +208,8 @@ arch_archive_mirror (int chatter_fd,
 static void
 mirror_revision (struct arch_archive * arch,
                  struct arch_archive * to_arch,
-                 t_uchar * revision,
-                 t_uchar * prev_level,
+                 const t_uchar * revision,
+                 const t_uchar * prev_level,
                  struct arch_archive_mirror_options * archive_mirror_opts)
 {
   t_uchar * version = 0;
@@ -332,7 +332,7 @@ mirror_revision (struct arch_archive * arch,
       assoc_table headers = 0;
       t_uchar * body = 0;
       
-      arch_parse_log (0, &headers, &body, log);
+      arch_parse_log (0, &headers, (const t_uchar **)&body, log);
       arch_print_headers_summary (1, 4, headers, arch_include_summary);
       lim_free (0, log);
       free_assoc_table (headers);
@@ -382,13 +382,13 @@ mirror_revision (struct arch_archive * arch,
 
 
 static int
-list_contains (rel_table list, t_uchar * elt)
+list_contains (rel_table list, const t_uchar * elt)
 {
   int x;
 
   for (x = 0; x < rel_n_records (list); ++x)
     {
-      if (!str_cmp (elt, list[x][0]))
+      if (!str_cmp (elt, rel_peek_str (list, x, 0)))
         return 1;
     }
   return 0;
