@@ -36,93 +36,131 @@
 static void pfs_close (struct arch_archive * a);
 static t_uchar * pfs_archive_version (struct arch_archive * a);
 static rel_table pfs_categories (struct arch_archive * a);
-static rel_table pfs_branches (struct arch_archive * a, t_uchar * category);
-static rel_table pfs_versions (struct arch_archive * a, t_uchar * package);
-static rel_table pfs_revisions (struct arch_archive * a, t_uchar * version);
-static t_uchar * pfs_archive_log (struct arch_archive * a, t_uchar * revision);
-static int pfs_revision_type  (enum arch_revision_type * type, int * archive_cached,
-                                struct arch_archive * a, t_uchar * revision);
-static int list_contains (rel_table files, t_uchar * name);
-static void pfs_get_patch (int out_fd, struct arch_archive * a, t_uchar * revision);
-static void pfs_get_cached (int out_fd, struct arch_archive * a, t_uchar * revision);
-static void pfs_get_import (int out_fd, struct arch_archive * a, t_uchar * revision);
-static t_uchar * pfs_get_continuation (struct arch_archive * a, t_uchar * revision);
-static t_uchar * pfs_get_meta_info (struct arch_archive * a, t_uchar * meta_info_name);
-static int pfs_make_category (t_uchar ** errstr, struct arch_archive * a, t_uchar * category);
-static int pfs_make_branch (t_uchar ** errstr, struct arch_archive * a, t_uchar * branch);
-static int pfs_make_version (t_uchar ** errstr, struct arch_archive * a, t_uchar * version);
+static rel_table pfs_branches (struct arch_archive * a,
+                               const t_uchar * category);
+static rel_table pfs_versions (struct arch_archive * a,
+                               const t_uchar * package);
+static rel_table pfs_revisions (struct arch_archive * a,
+                                const t_uchar * version);
+static t_uchar * pfs_archive_log (struct arch_archive * a,
+                                  const t_uchar * revision);
+static int pfs_revision_type (enum arch_revision_type * type,
+                              int * archive_cached,
+                              struct arch_archive * a,
+                              const t_uchar * revision);
+static int list_contains (rel_table files,
+                          const t_uchar * name);
+static void pfs_get_patch (int out_fd,
+                           struct arch_archive * a,
+                           const t_uchar * revision);
+static void pfs_get_cached (int out_fd,
+                            struct arch_archive * a,
+                            const t_uchar * revision);
+static void pfs_get_import (int out_fd,
+                            struct arch_archive * a,
+                            const t_uchar * revision);
+static t_uchar * pfs_get_continuation (struct arch_archive * a,
+                                       const t_uchar * revision);
+static t_uchar * pfs_get_meta_info (struct arch_archive * a,
+                                    const t_uchar * meta_info_name);
+static int pfs_make_category (t_uchar ** errstr,
+                              struct arch_archive * a,
+                              const t_uchar * category);
+static int pfs_make_branch (t_uchar ** errstr,
+                            struct arch_archive * a,
+                            const t_uchar * branch);
+static int pfs_make_version (t_uchar ** errstr,
+                             struct arch_archive * a,
+                             const t_uchar * version);
 static int pfs_lock_revision (t_uchar ** errstr, struct arch_archive * a,
-                              t_uchar * version,
-                              t_uchar * prev_level,
-                              t_uchar * uid,
-                              t_uchar * txn_id,
-                              t_uchar * new_level);
+                              const t_uchar * version,
+                              const t_uchar * prev_level,
+                              const t_uchar * uid,
+                              const t_uchar * txn_id,
+                              const t_uchar * new_level);
 static int pfs_revision_ready (t_uchar ** errstr, struct arch_archive * a,
-                               t_uchar * version,
-                               t_uchar * prev_level,
-                               t_uchar * uid,
-                               t_uchar * txn_id,
-                               t_uchar * new_level);
+                               const t_uchar * version,
+                               const t_uchar * prev_level,
+                               const t_uchar * uid,
+                               const t_uchar * txn_id,
+                               const t_uchar * new_level);
 static int pfs_finish_revision (t_uchar ** errstr, struct arch_archive * a,
-                                t_uchar * version,
-                                t_uchar * prev_level,
-                                t_uchar * uid,
-                                t_uchar * txn_id,
-                                t_uchar * new_level);
+                                const t_uchar * version,
+                                const t_uchar * prev_level,
+                                const t_uchar * uid,
+                                const t_uchar * txn_id,
+                                const t_uchar * new_level);
 static enum arch_revision_lock_state pfs_lock_state (t_uchar ** prev_level_ret,
                                                      t_uchar ** uid_ret,
                                                      t_uchar ** txn_id_ret,
                                                      struct arch_archive * a,
-                                                     t_uchar * version);
+                                                     const t_uchar * version);
 static int pfs_break_revision_lock (t_uchar ** errstr, struct arch_archive * a,
-                                    t_uchar * version,
-                                    t_uchar * prev_level,
-                                    t_uchar * uid,
-                                    t_uchar * txn_id);
-static void safely_remove_stale_broken_lock_dir (struct arch_pfs_archive * arch, t_uchar * broken_dir, t_uchar * prev_level);
-static void safely_remove_spent_lock (struct arch_pfs_archive * arch, t_uchar * spent_lock);
+                                    const t_uchar * version,
+                                    const t_uchar * prev_level,
+                                    const t_uchar * uid,
+                                    const t_uchar * txn_id);
+static void safely_remove_stale_broken_lock_dir (struct arch_pfs_archive * arch,
+                                                 const t_uchar * broken_dir,
+                                                 const t_uchar * prev_level);
+static void safely_remove_spent_lock (struct arch_pfs_archive * arch,
+                                      const t_uchar * spent_lock);
 static int pfs_put_log (t_uchar ** errstr, struct arch_archive * a,
-                        t_uchar * version,
-                        t_uchar * prev_level,
-                        t_uchar * uid,
-                        t_uchar * txn_id,
-                        t_uchar * log_text);
+                        const t_uchar * version,
+                        const t_uchar * prev_level,
+                        const t_uchar * uid,
+                        const t_uchar * txn_id,
+                        const t_uchar * log_text);
 static int pfs_put_continuation (t_uchar ** errstr, struct arch_archive * a,
-                                 t_uchar * version,
-                                 t_uchar * prev_level,
-                                 t_uchar * uid,
-                                 t_uchar * txn_id,
-                                 t_uchar * continuation);
+                                 const t_uchar * version,
+                                 const t_uchar * prev_level,
+                                 const t_uchar * uid,
+                                 const t_uchar * txn_id,
+                                 const t_uchar * continuation);
 static int pfs_put_changeset (t_uchar ** errstr, struct arch_archive * a,
-                              t_uchar * version,
-                              t_uchar * prev_level,
-                              t_uchar * uid,
-                              t_uchar * txn_id,
-                              t_uchar * level,
+                              const t_uchar * version,
+                              const t_uchar * prev_level,
+                              const t_uchar * uid,
+                              const t_uchar * txn_id,
+                              const t_uchar * level,
                               int in_fd);
 static int pfs_put_import (t_uchar ** errstr, struct arch_archive * a,
-                           t_uchar * version,
-                           t_uchar * prev_level,
-                           t_uchar * uid,
-                           t_uchar * txn_id,
-                           t_uchar * level,
+                           const t_uchar * version,
+                           const t_uchar * prev_level,
+                           const t_uchar * uid,
+                           const t_uchar * txn_id,
+                           const t_uchar * level,
                            int in_fd);
-static int pfs_put_cached (t_uchar ** errstr, struct arch_archive * a, t_uchar * revision, int in_fd);
-static int pfs_delete_cached (t_uchar ** errstr, struct arch_archive * a, t_uchar * revision);
-static void pfs_repair_non_txnal (int chatter_fd, struct arch_archive * a);
-static void sign_and_upload (struct arch_archive * a, t_uchar * tmp_file, t_uchar * path, int in_fd, t_uchar * description, int batteries_to_power);
+static int pfs_put_cached (t_uchar ** errstr,
+                           struct arch_archive * a,
+                           const t_uchar * revision,
+                           int in_fd);
+static int pfs_delete_cached (t_uchar ** errstr,
+                              struct arch_archive * a,
+                              const t_uchar * revision);
+static void pfs_repair_non_txnal (int chatter_fd,
+                                  struct arch_archive * a);
+static void sign_and_upload (struct arch_archive * a,
+                             const t_uchar * tmp_file,
+                             const t_uchar * path,
+                             int in_fd,
+                             const t_uchar * description,
+                             int batteries_to_power);
 static void pfs_create_signature_file (struct arch_archive * a,
-                                       t_uchar * revision);
+                                       const t_uchar * revision);
 static void pfs_finish_signature_file (struct arch_archive *a, 
-                                       t_uchar * version,
-                                       t_uchar * prev_level,
-                                       t_uchar * uid,
-                                       t_uchar * txn_id,
-                                       t_uchar * new_level);
-static void pfs_finish_signature_file_worker (struct arch_archive * a, t_uchar * revision, t_uchar * dir, t_uchar * checksumname);
+                                       const t_uchar * version,
+                                       const t_uchar * prev_level,
+                                       const t_uchar * uid,
+                                       const t_uchar * txn_id,
+                                       const t_uchar * new_level);
+static void pfs_finish_signature_file_worker (struct arch_archive * a,
+                                              const t_uchar * revision,
+                                              const t_uchar * dir,
+                                              const t_uchar * checksumname);
 static void run_md5sum (int signature_file,
-                        t_uchar * description, 
-                        t_uchar * data_file);
+                        const t_uchar * description, 
+                        const t_uchar * data_file);
 
 
 
@@ -172,11 +210,17 @@ static struct arch_archive_vtable pfs_vtable =
 
 
 void
-arch_pfs_make_archive (t_uchar * name, t_uchar *location, t_uchar * version, t_uchar * mirror_of, int dot_listing_lossage, int signed_archive)
+arch_pfs_make_archive (const t_uchar * name,
+                       const t_uchar *location,
+                       const t_uchar * version,
+                       const t_uchar * mirror_of,
+                       int dot_listing_lossage,
+                       int signed_archive)
 {
   arch_pfs_pfs_make_archive (name, location, version, mirror_of, dot_listing_lossage, signed_archive);
 }
 
+
 void
 arch_pfs_archive_connect (struct arch_archive ** a)
 {
@@ -243,8 +287,8 @@ static rel_table
 pfs_categories (struct arch_archive * a)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
-  rel_table files = 0;
-  rel_table answer = 0;
+  rel_table files  = rel_table_nil;
+  rel_table answer  = rel_table_nil;
 
   files = arch_pfs_directory_files (arch->pfs, ".", 0);
   answer = arch_pick_categories_by_field (files, 0);
@@ -254,14 +298,14 @@ pfs_categories (struct arch_archive * a)
 }
 
 
-
 static rel_table
-pfs_branches (struct arch_archive * a, t_uchar * category)
+pfs_branches (struct arch_archive * a,
+              const t_uchar * category)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * category_path = 0;
-  rel_table files = 0;
-  rel_table answer = 0;
+  rel_table files  = rel_table_nil;
+  rel_table answer  = rel_table_nil;
 
   category_path = arch_fs_archive_category_path (0, category);
 
@@ -274,12 +318,13 @@ pfs_branches (struct arch_archive * a, t_uchar * category)
 
 
 static rel_table
-pfs_versions (struct arch_archive * a, t_uchar * package)
+pfs_versions (struct arch_archive * a,
+              const t_uchar * package)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * branch_path = 0;
-  rel_table files = 0;
-  rel_table answer = 0;
+  rel_table files  = rel_table_nil;
+  rel_table answer  = rel_table_nil;
 
   branch_path = arch_fs_archive_branch_path (0, package);
 
@@ -291,14 +336,14 @@ pfs_versions (struct arch_archive * a, t_uchar * package)
 }
 
 
-
 static rel_table
-pfs_revisions (struct arch_archive * a, t_uchar * version)
+pfs_revisions (struct arch_archive * a,
+               const t_uchar * version)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * version_path = 0;
-  rel_table files = 0;
-  rel_table answer = 0;
+  rel_table files  = rel_table_nil;
+  rel_table answer  = rel_table_nil;
 
   version_path = arch_fs_archive_version_path (0, version);
 
@@ -311,7 +356,8 @@ pfs_revisions (struct arch_archive * a, t_uchar * version)
 
 
 static t_uchar *
-pfs_archive_log (struct arch_archive * a, t_uchar * revision)
+pfs_archive_log (struct arch_archive * a,
+                 const t_uchar * revision)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * revision_log_path = 0;
@@ -333,8 +379,10 @@ pfs_archive_log (struct arch_archive * a, t_uchar * revision)
 
 
 static int
-pfs_revision_type (enum arch_revision_type * type, int * archive_cached,
-                   struct arch_archive * a, t_uchar * revision)
+pfs_revision_type (enum arch_revision_type * type,
+                   int * archive_cached,
+                   struct arch_archive * a,
+                   const t_uchar * revision)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * continuation_path = 0;
@@ -348,7 +396,7 @@ pfs_revision_type (enum arch_revision_type * type, int * archive_cached,
   t_uchar * cacherev_tail = 0;
   t_uchar * log_tail = 0;
   t_uchar * revision_dir = 0;
-  rel_table files = 0;
+  rel_table files  = rel_table_nil;
   int has_continuation_file;
   int has_changeset_file;
   int has_import_file;
@@ -370,7 +418,7 @@ pfs_revision_type (enum arch_revision_type * type, int * archive_cached,
 
   revision_dir = file_name_directory_file (0, continuation_path);
   files = arch_pfs_directory_files (arch->pfs, revision_dir, 1);
-  if (files == 0) 
+  if (!rel_n_records (files))
   {
     result = -1;
     goto pfs_revision_type_cleanup;
@@ -460,13 +508,14 @@ pfs_revision_type (enum arch_revision_type * type, int * archive_cached,
 }
 
 static int
-list_contains (rel_table files, t_uchar * name)
+list_contains (rel_table files,
+               const t_uchar * name)
 {
   int x;
 
   for (x = 0; x < rel_n_records (files); ++x)
     {
-      if (!str_cmp (files[x][0], name))
+      if (!str_cmp (rel_peek_str (files, x, 0), name))
         return 1;
     }
 
@@ -475,7 +524,9 @@ list_contains (rel_table files, t_uchar * name)
 
 
 static void
-pfs_get_patch (int out_fd, struct arch_archive * a, t_uchar * revision)
+pfs_get_patch (int out_fd,
+               struct arch_archive * a,
+               const t_uchar * revision)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * changeset_path = 0;
@@ -494,7 +545,9 @@ pfs_get_patch (int out_fd, struct arch_archive * a, t_uchar * revision)
 
 
 static void
-pfs_get_cached (int out_fd, struct arch_archive * a, t_uchar * revision)
+pfs_get_cached (int out_fd,
+                struct arch_archive * a,
+                const t_uchar * revision)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * cached_path = 0;
@@ -512,9 +565,10 @@ pfs_get_cached (int out_fd, struct arch_archive * a, t_uchar * revision)
 }
 
 
-
 static void
-pfs_get_import (int out_fd, struct arch_archive * a, t_uchar * revision)
+pfs_get_import (int out_fd,
+                struct arch_archive * a,
+                const t_uchar * revision)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * import_path = 0;
@@ -533,7 +587,8 @@ pfs_get_import (int out_fd, struct arch_archive * a, t_uchar * revision)
 
 
 static t_uchar *
-pfs_get_continuation (struct arch_archive * a, t_uchar * revision)
+pfs_get_continuation (struct arch_archive * a,
+                      const t_uchar * revision)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * continuation_path = 0;
@@ -553,8 +608,10 @@ pfs_get_continuation (struct arch_archive * a, t_uchar * revision)
   return answer;
 }
 
+
 static t_uchar *
-pfs_get_meta_info (struct arch_archive * a, t_uchar * meta_info_name)
+pfs_get_meta_info (struct arch_archive * a,
+                   const t_uchar * meta_info_name)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * meta_info_path = 0;
@@ -573,7 +630,9 @@ pfs_get_meta_info (struct arch_archive * a, t_uchar * meta_info_name)
 
 
 static int
-pfs_make_category (t_uchar ** errstr, struct arch_archive * a, t_uchar * category)
+pfs_make_category (t_uchar ** errstr,
+                   struct arch_archive * a,
+                   const t_uchar * category)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * cat_path = 0;
@@ -638,9 +697,10 @@ pfs_make_category (t_uchar ** errstr, struct arch_archive * a, t_uchar * categor
 }
 
 
-
 static int
-pfs_make_branch (t_uchar ** errstr, struct arch_archive * a, t_uchar * branch)
+pfs_make_branch (t_uchar ** errstr,
+                 struct arch_archive * a,
+                 const t_uchar * branch)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * branch_path = 0;
@@ -694,9 +754,10 @@ pfs_make_branch (t_uchar ** errstr, struct arch_archive * a, t_uchar * branch)
 }
 
 
-
 static int
-pfs_make_version (t_uchar ** errstr, struct arch_archive * a, t_uchar * version)
+pfs_make_version (t_uchar ** errstr,
+                  struct arch_archive * a,
+                  const t_uchar * version)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * version_path = 0;
@@ -865,11 +926,11 @@ pfs_make_version (t_uchar ** errstr, struct arch_archive * a, t_uchar * version)
 
 static int
 pfs_lock_revision (t_uchar ** errstr, struct arch_archive * a,
-                   t_uchar * version,
-                   t_uchar * prev_level,
-                   t_uchar * uid,
-                   t_uchar * txn_id,
-                   t_uchar * new_level)
+                   const t_uchar * version,
+                   const t_uchar * prev_level,
+                   const t_uchar * uid,
+                   const t_uchar * txn_id,
+                   const t_uchar * new_level)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * unlocked_path = 0;
@@ -989,11 +1050,11 @@ pfs_lock_revision (t_uchar ** errstr, struct arch_archive * a,
 
 static int
 pfs_revision_ready (t_uchar ** errstr, struct arch_archive * a,
-                    t_uchar * version,
-                    t_uchar * prev_level,
-                    t_uchar * uid,
-                    t_uchar * txn_id,
-                    t_uchar * new_level)
+                    const t_uchar * version,
+                    const t_uchar * prev_level,
+                    const t_uchar * uid,
+                    const t_uchar * txn_id,
+                    const t_uchar * new_level)
 {
   pfs_finish_signature_file (a, version, prev_level, uid, txn_id, new_level);
   return 0;
@@ -1002,11 +1063,11 @@ pfs_revision_ready (t_uchar ** errstr, struct arch_archive * a,
 
 static int
 pfs_finish_revision (t_uchar ** errstr, struct arch_archive * a,
-                     t_uchar * version,
-                     t_uchar * prev_level,
-                     t_uchar * uid,
-                     t_uchar * txn_id,
-                     t_uchar * new_level)
+                     const t_uchar * version,
+                     const t_uchar * prev_level,
+                     const t_uchar * uid,
+                     const t_uchar * txn_id,
+                     const t_uchar * new_level)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * locked_path = 0;
@@ -1103,14 +1164,14 @@ pfs_finish_revision (t_uchar ** errstr, struct arch_archive * a,
 
 static enum arch_revision_lock_state
 pfs_lock_state (t_uchar ** prev_level_ret,
-               t_uchar ** uid_ret,
-               t_uchar ** txn_id_ret,
-               struct arch_archive * a,
-               t_uchar * version)
+                t_uchar ** uid_ret,
+                t_uchar ** txn_id_ret,
+                struct arch_archive * a,
+                const t_uchar * version)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
 
-  rel_table revisions = 0;
+  rel_table revisions  = rel_table_nil;
   t_uchar * prev_level = 0;
 
   t_uchar * unlocked_path = 0;
@@ -1123,10 +1184,10 @@ pfs_lock_state (t_uchar ** prev_level_ret,
 
 
   revisions = arch_archive_revisions (&arch->arch, version, 0);
-  if (!revisions)
+  if (!rel_n_records (revisions))
     prev_level = 0;
   else
-    prev_level = str_save (0, revisions[rel_n_records (revisions) - 1][0]);
+    prev_level = str_save (0, rel_peek_str (revisions, rel_n_records (revisions) - 1, 0));
 
   if (prev_level_ret)
     *prev_level_ret = prev_level ? str_save (0, prev_level) : 0;
@@ -1148,7 +1209,7 @@ pfs_lock_state (t_uchar ** prev_level_ret,
   else
     {
       t_uchar * version_path = 0;
-      rel_table files = 0;
+      rel_table files  = rel_table_nil;
       t_uchar * persistent_locked_path_stem = 0;
       int x;
 
@@ -1166,12 +1227,12 @@ pfs_lock_state (t_uchar ** prev_level_ret,
 
       for (x = 0; x < rel_n_records (files); ++x)
         {
-          if (!str_cmp_prefix (persistent_locked_path_stem, files[x][0]))
+          if (!str_cmp_prefix (persistent_locked_path_stem, rel_peek_str (files, x, 0)))
             {
               t_uchar * contents_rel = 0;
               t_uchar * contents_path = 0;
 
-              contents_rel = file_name_in_vicinity (0, files[x][0], "+contents");
+              contents_rel = file_name_in_vicinity (0, rel_peek_str (files, x, 0), "+contents");
               contents_path = file_name_in_vicinity (0, version_path, contents_rel);
 
               if (1 != arch_pfs_is_dir (arch->pfs, contents_path))
@@ -1180,10 +1241,10 @@ pfs_lock_state (t_uchar ** prev_level_ret,
                 }
               else
                 {
-                  t_uchar * uid_start;
-                  t_uchar * uid_end;
+                  const t_uchar * uid_start;
+                  const t_uchar * uid_end;
 
-                  uid_start = files[x][0] + str_length (persistent_locked_path_stem);
+                  uid_start = rel_peek_str (files, x, 0) + str_length (persistent_locked_path_stem);
                   uid_end = str_chr_rindex (uid_start, '-');
                   invariant (!!uid_end);
                   --uid_end;
@@ -1198,7 +1259,7 @@ pfs_lock_state (t_uchar ** prev_level_ret,
                     }
                   else
                     {
-                      t_uchar * txn_id;
+                      const t_uchar * txn_id;
 
                       txn_id = uid_end + 2;
                       if (uid_ret)
@@ -1235,10 +1296,10 @@ pfs_lock_state (t_uchar ** prev_level_ret,
 
 static int
 pfs_break_revision_lock (t_uchar ** errstr, struct arch_archive * a,
-                        t_uchar * version,
-                        t_uchar * prev_level,
-                        t_uchar * uid,
-                        t_uchar * txn_id)
+                         const t_uchar * version,
+                         const t_uchar * prev_level,
+                         const t_uchar * uid,
+                         const t_uchar * txn_id)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * locked_path = 0;
@@ -1315,11 +1376,11 @@ pfs_break_revision_lock (t_uchar ** errstr, struct arch_archive * a,
                */
 
               {
-                rel_table levels = 0;
+                rel_table levels  = rel_table_nil;
 
                 levels = arch_archive_revisions (&arch->arch, version, 0);
-                if (levels)
-                  latest_rev = str_save (0, levels[rel_n_records(levels) - 1][0]);
+                if (rel_n_records (levels))
+                  latest_rev = str_save (0, rel_peek_str (levels, rel_n_records(levels) - 1, 0));
 
                 rel_free_table (levels);
               }
@@ -1366,12 +1427,13 @@ pfs_break_revision_lock (t_uchar ** errstr, struct arch_archive * a,
 }
 
 
-
 static void
-safely_remove_stale_broken_lock_dir (struct arch_pfs_archive * arch, t_uchar * broken_dir, t_uchar * prev_level)
+safely_remove_stale_broken_lock_dir (struct arch_pfs_archive * arch,
+                                     const t_uchar * broken_dir,
+                                     const t_uchar * prev_level)
 {
   t_uchar * broken_dir_tail = 0;
-  rel_table nested = 0;
+  rel_table nested  = rel_table_nil;
   int x;
   t_uchar * precious_basename = 0;
 
@@ -1383,9 +1445,9 @@ safely_remove_stale_broken_lock_dir (struct arch_pfs_archive * arch, t_uchar * b
 
   for (x = 0; x < rel_n_records (nested); ++x)
     {
-      t_uchar * basename;
+      const t_uchar * basename;
 
-      basename = nested[x][0];
+      basename = rel_peek_str (nested, x, 0);
 
       if (str_cmp (".", basename) && str_cmp ("..", basename) && str_cmp (precious_basename, basename))
         {
@@ -1407,10 +1469,11 @@ safely_remove_stale_broken_lock_dir (struct arch_pfs_archive * arch, t_uchar * b
 
 
 static void
-safely_remove_spent_lock (struct arch_pfs_archive * arch, t_uchar * spent_lock)
+safely_remove_spent_lock (struct arch_pfs_archive * arch,
+                          const t_uchar * spent_lock)
 {
   t_uchar * spent_lock_tail = 0;
-  rel_table nested = 0;
+  rel_table nested  = rel_table_nil;
   t_uchar * tmp_path = 0;
   int x;
 
@@ -1425,9 +1488,9 @@ safely_remove_spent_lock (struct arch_pfs_archive * arch, t_uchar * spent_lock)
 
   for (x = 0; x < rel_n_records (nested); ++x)
     {
-      t_uchar * basename;
+      const t_uchar * basename;
 
-      basename = nested[x][0];
+      basename = rel_peek_str (nested, x, 0);
 
       if (str_cmp (".", basename) && str_cmp ("..", basename) && str_cmp ("+contents", basename))
         {
@@ -1457,11 +1520,11 @@ safely_remove_spent_lock (struct arch_pfs_archive * arch, t_uchar * spent_lock)
 
 static int
 pfs_put_log (t_uchar ** errstr, struct arch_archive * a,
-             t_uchar * version,
-             t_uchar * prev_level,
-             t_uchar * uid,
-             t_uchar * txn_id,
-             t_uchar * log_text)
+             const t_uchar * version,
+             const t_uchar * prev_level,
+             const t_uchar * uid,
+             const t_uchar * txn_id,
+             const t_uchar * log_text)
 {
   t_uchar * locked_contents_path = 0;
   t_uchar * log_path = 0;
@@ -1498,11 +1561,11 @@ pfs_put_log (t_uchar ** errstr, struct arch_archive * a,
 
 static int
 pfs_put_continuation (t_uchar ** errstr, struct arch_archive * a,
-                     t_uchar * version,
-                     t_uchar * prev_level,
-                     t_uchar * uid,
-                     t_uchar * txn_id,
-                     t_uchar * continuation)
+                      const t_uchar * version,
+                      const t_uchar * prev_level,
+                      const t_uchar * uid,
+                      const t_uchar * txn_id,
+                      const t_uchar * continuation)
 {
   t_uchar * locked_contents_path = 0;
   t_uchar * continuation_path = 0;
@@ -1538,12 +1601,12 @@ pfs_put_continuation (t_uchar ** errstr, struct arch_archive * a,
 
 static int
 pfs_put_changeset (t_uchar ** errstr, struct arch_archive * a,
-                  t_uchar * version,
-                  t_uchar * prev_level,
-                  t_uchar * uid,
-                  t_uchar * txn_id,
-                  t_uchar * level,
-                  int in_fd)
+                   const t_uchar * version,
+                   const t_uchar * prev_level,
+                   const t_uchar * uid,
+                   const t_uchar * txn_id,
+                   const t_uchar * level,
+                   int in_fd)
 {
   t_uchar * locked_contents_path = 0;
   t_uchar * patches = 0;
@@ -1567,14 +1630,15 @@ pfs_put_changeset (t_uchar ** errstr, struct arch_archive * a,
   return 0;
 }
 
+
 static int
 pfs_put_import (t_uchar ** errstr, struct arch_archive * a,
-               t_uchar * version,
-               t_uchar * prev_level,
-               t_uchar * uid,
-               t_uchar * txn_id,
-               t_uchar * level,
-               int in_fd)
+                const t_uchar * version,
+                const t_uchar * prev_level,
+                const t_uchar * uid,
+                const t_uchar * txn_id,
+                const t_uchar * level,
+                int in_fd)
 {
   t_uchar * locked_contents_path = 0;
   t_uchar * import = 0;
@@ -1600,7 +1664,10 @@ pfs_put_import (t_uchar ** errstr, struct arch_archive * a,
 
 
 static int
-pfs_put_cached (t_uchar ** errstr, struct arch_archive * a, t_uchar * revision, int in_fd)
+pfs_put_cached (t_uchar ** errstr,
+                struct arch_archive * a,
+                const t_uchar * revision,
+                int in_fd)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * cached_path = 0;
@@ -1629,7 +1696,9 @@ pfs_put_cached (t_uchar ** errstr, struct arch_archive * a, t_uchar * revision,
 
 
 static int
-pfs_delete_cached (t_uchar ** errstr, struct arch_archive * a, t_uchar * revision)
+pfs_delete_cached (t_uchar ** errstr,
+                   struct arch_archive * a,
+                   const t_uchar * revision)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * cached_path = 0;
@@ -1662,11 +1731,12 @@ pfs_delete_cached (t_uchar ** errstr, struct arch_archive * a, t_uchar * revisio
 
 
 static void
-pfs_repair_non_txnal (int chatter_fd, struct arch_archive * a)
+pfs_repair_non_txnal (int chatter_fd,
+                      struct arch_archive * a)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * meta_info_path = 0;
-  rel_table categories = 0;
+  rel_table categories  = rel_table_nil;
   int c;
 
   if (!a->http_blows)
@@ -1684,48 +1754,48 @@ pfs_repair_non_txnal (int chatter_fd, struct arch_archive * a)
   for (c = 0; c < rel_n_records (categories); ++c)
     {
       t_uchar * cat_path = 0;
-      rel_table branches = 0;
+      rel_table branches  = rel_table_nil;
       int b;
 
-      arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for category %s\n", categories[c][0]);
+      arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for category %s\n", rel_peek_str (categories, c, 0));
 
-      cat_path = arch_fs_archive_category_path (0, categories[c][0]);
+      cat_path = arch_fs_archive_category_path (0, rel_peek_str (categories, c, 0));
       arch_pfs_update_listing_file (arch->pfs, cat_path);
 
-      branches = pfs_branches (a, categories[c][0]);
+      branches = pfs_branches (a, rel_peek_str (categories, c, 0));
 
       for (b = 0; b < rel_n_records (branches); ++b)
         {
           t_uchar * branch_path = 0;
-          rel_table versions = 0;
+          rel_table versions  = rel_table_nil;
           int v;
 
-          arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for branch %s\n", branches[b][0]);
+          arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for branch %s\n", rel_peek_str (branches, b, 0));
 
-          branch_path = arch_fs_archive_branch_path (0, branches[b][0]);
+          branch_path = arch_fs_archive_branch_path (0, rel_peek_str (branches, b, 0));
           arch_pfs_update_listing_file (arch->pfs, branch_path);
 
-          versions = pfs_versions (a, branches[b][0]);
+          versions = pfs_versions (a, rel_peek_str (branches, b, 0));
 
           for (v = 0; v < rel_n_records (versions); ++v)
             {
               t_uchar * version_path = 0;
-              rel_table revisions = 0;
+              rel_table revisions  = rel_table_nil;
               int r;
 
-              arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for version %s\n", versions[v][0]);
+              arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for version %s\n", rel_peek_str (versions, v, 0));
 
-              version_path = arch_fs_archive_version_path (0, versions[v][0]);
+              version_path = arch_fs_archive_version_path (0, rel_peek_str (versions, v, 0));
               arch_pfs_update_listing_file (arch->pfs, version_path);
 
-              revisions = pfs_revisions (a, versions[v][0]);
+              revisions = pfs_revisions (a, rel_peek_str (versions, v, 0));
 
               for (r = 0; r < rel_n_records (revisions); ++r)
                 {
                   t_uchar * package_revision = 0;
                   t_uchar * revision_path = 0;
 
-                  package_revision = str_alloc_cat_many (0, versions[v][0], "--", revisions[r][0], str_end);
+                  package_revision = str_alloc_cat_many (0, rel_peek_str (versions, v, 0), "--", rel_peek_str (revisions, r, 0), str_end);
                   arch_chatter (chatter_fd, "* archive-fixup: rebuilding index file for revisions %s\n", package_revision);
 
                   revision_path = arch_fs_archive_revision_path (0, package_revision);
@@ -1757,7 +1827,12 @@ pfs_repair_non_txnal (int chatter_fd, struct arch_archive * a)
 
 
 static void
-sign_and_upload (struct arch_archive * a, t_uchar * tmp_file, t_uchar * path, int in_fd, t_uchar * description, int batteries_to_power)
+sign_and_upload (struct arch_archive * a,
+                 const t_uchar * tmp_file,
+                 const t_uchar * path,
+                 int in_fd,
+                 const t_uchar * description,
+                 int batteries_to_power)
 {
   struct arch_pfs_archive *arch = (struct arch_pfs_archive *)a;
   
@@ -1796,7 +1871,7 @@ sign_and_upload (struct arch_archive * a, t_uchar * tmp_file, t_uchar * path, in
 
 static void
 pfs_create_signature_file (struct arch_archive * a,
-                           t_uchar * revision)
+                           const t_uchar * revision)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
 
@@ -1819,11 +1894,11 @@ pfs_create_signature_file (struct arch_archive * a,
 
 static void
 pfs_finish_signature_file (struct arch_archive *a, 
-                           t_uchar * version,
-                           t_uchar * prev_level,
-                           t_uchar * uid,
-                           t_uchar * txn_id,
-                           t_uchar * new_level)
+                           const t_uchar * version,
+                           const t_uchar * prev_level,
+                           const t_uchar * uid,
+                           const t_uchar * txn_id,
+                           const t_uchar * new_level)
 {
   struct arch_pfs_archive * arch = (struct arch_pfs_archive *)a;
   t_uchar * dir = 0;
@@ -1843,7 +1918,10 @@ pfs_finish_signature_file (struct arch_archive *a,
 
 
 static void
-pfs_finish_signature_file_worker (struct arch_archive * a, t_uchar * revision, t_uchar * dir, t_uchar * checksumname)
+pfs_finish_signature_file_worker (struct arch_archive * a,
+                                  const t_uchar * revision,
+                                  const t_uchar * dir,
+                                  const t_uchar * checksumname)
 {
   struct arch_pfs_archive *arch = (struct arch_pfs_archive *)a;
   t_uchar * signed_file = 0;
@@ -1918,8 +1996,8 @@ pfs_finish_signature_file_worker (struct arch_archive * a, t_uchar * revision, t
 
 static void
 run_md5sum (int signature_file,
-	    t_uchar * description, 
-            t_uchar * data_file)
+	    const t_uchar * description, 
+            const t_uchar * data_file)
 {
   md5_context_t md5c = 0;
   t_uchar md5[16];
