@@ -30,7 +30,6 @@
 #include "hackerlab/fs/file-names.h"
 #include "tla/libfsutils/tmp-files.h"
 #include "tla/libfsutils/rmrf.h"
-#include "tla/libawk/trim.h"
 #include "tla/libarch/namespace.h"
 #include "tla/libarch/my.h"
 #include "tla/libarch/exec.h"
@@ -44,7 +43,8 @@
 /* __STDC__ prototypes for static functions */
 static int invoke_tar_extract (int * pid_ret);
 static int wait_for_tar (int pid);
-static void ensure_writable (struct arch_archive * arch, int mirror_ok);
+static void ensure_writable (struct arch_archive * arch,
+                             int mirror_ok);
 
 
 
@@ -53,7 +53,11 @@ static struct arch_archive * connected_by_name = 0;
 
 
 void
-arch_make_archive (t_uchar * name, t_uchar * location, t_uchar * mirror_of, int dot_listing_lossage, int signed_archive)
+arch_make_archive (const t_uchar * name,
+                   const t_uchar * location,
+                   const t_uchar * mirror_of,
+                   int dot_listing_lossage,
+                   int signed_archive)
 {
   t_uchar * current_loc = 0;
   t_uchar * version;
@@ -79,7 +83,9 @@ arch_make_archive (t_uchar * name, t_uchar * location, t_uchar * mirror_of, int
 
 
 struct arch_archive *
-arch_archive_connect_location (t_uchar * name, t_uchar * location, t_uchar * want_mirror_of)
+arch_archive_connect_location (const t_uchar * name,
+                               const t_uchar * location,
+                               const t_uchar * want_mirror_of)
 {
   struct arch_archive * answer = 0;
 
@@ -102,8 +108,8 @@ arch_archive_connect_location (t_uchar * name, t_uchar * location, t_uchar * wan
       exit (2);
     }
 
-  answer->mirror_of = trim_surrounding_ws (arch_get_meta_info (answer, "mirror"));
-  answer->official_name = trim_surrounding_ws (arch_get_meta_info (answer, "name"));
+  answer->mirror_of = arch_get_meta_info_trimming (answer, "mirror");
+  answer->official_name = arch_get_meta_info_trimming (answer, "name");
   if (!answer->official_name)
     {
       int ign;
@@ -128,7 +134,8 @@ arch_archive_connect_location (t_uchar * name, t_uchar * location, t_uchar * wan
 
 
 struct arch_archive *
-arch_archive_connect (t_uchar * name, t_uchar * want_mirror_of)
+arch_archive_connect (const t_uchar * name,
+                      const t_uchar * want_mirror_of)
 {
   struct arch_archive * answer = 0;
 
@@ -210,7 +217,7 @@ arch_archive_version (struct arch_archive * arch)
 rel_table
 arch_archive_categories (struct arch_archive * arch)
 {
-  rel_table answer = 0;
+  rel_table answer  = rel_table_nil;
 
   answer = arch->vtable->categories (arch);
   rel_sort_table_by_field (0, answer, 0);
@@ -220,9 +227,10 @@ arch_archive_categories (struct arch_archive * arch)
 
 
 rel_table
-arch_archive_branches (struct arch_archive * arch, t_uchar * category)
+arch_archive_branches (struct arch_archive * arch,
+                       const t_uchar * category)
 {
-  rel_table answer = 0;
+  rel_table answer  = rel_table_nil;
 
   answer = arch->vtable->branches (arch, category);
   rel_sort_table_by_field (0, answer, 0);
@@ -232,9 +240,10 @@ arch_archive_branches (struct arch_archive * arch, t_uchar * category)
 
 
 rel_table
-arch_archive_versions (struct arch_archive * arch, t_uchar * package)
+arch_archive_versions (struct arch_archive * arch,
+                       const t_uchar * package)
 {
-  rel_table answer = 0;
+  rel_table answer  = rel_table_nil;
 
   answer = arch->vtable->versions (arch, package);
   arch_sort_table_by_name_field (0, answer, 0);
@@ -243,15 +252,17 @@ arch_archive_versions (struct arch_archive * arch, t_uchar * package)
 }
 
 t_uchar *
-arch_archive_latest_revision (struct arch_archive * arch, t_uchar * version, int full)
+arch_archive_latest_revision (struct arch_archive * arch,
+                              const t_uchar * version,
+                              int full)
 {
-  rel_table revs = 0;
+  rel_table revs  = rel_table_nil;
   t_uchar * answer = 0;
 
   revs = arch_archive_revisions (arch, version, full);
 
-  if (revs)
-    answer = str_save (0, revs[rel_n_records (revs) - 1][0]);
+  if (rel_n_records (revs))
+    answer = str_save (0, rel_peek_str (revs, rel_n_records (revs) - 1, 0));
 
   rel_free_table (revs);
   return answer;
@@ -259,9 +270,11 @@ arch_archive_latest_revision (struct arch_archive * arch, t_uchar * version, int
 
 
 rel_table
-arch_archive_revisions (struct arch_archive * arch, t_uchar * version, int full)
+arch_archive_revisions (struct arch_archive * arch,
+                        const t_uchar * version,
+                        int full)
 {
-  rel_table answer = 0;
+  rel_table answer  = rel_table_nil;
 
   answer = arch->vtable->revisions (arch, version);
   arch_sort_table_by_patch_level_field (0, answer, 0);
@@ -278,11 +291,13 @@ arch_archive_revisions (struct arch_archive * arch, t_uchar * version, int full)
 
       for (x = 0; x < rel_n_records (answer); ++x)
         {
-          t_uchar * t = 0;
+          const t_uchar * t;
+          t_uchar * new_t;
 
-          t = answer[x][0];
-          answer[x][0] = str_alloc_cat_many (0, fqv, "--", t, str_end);
-          lim_free (0, t);
+          t = rel_peek_str (answer, x, 0);
+          new_t = str_alloc_cat_many (0, fqv, "--", t, str_end);
+          rel_set_taking (answer, x, 0, rel_make_field_str (new_t));
+          lim_free (0, new_t);
         }
 
       lim_free (0, fqv);
@@ -292,7 +307,8 @@ arch_archive_revisions (struct arch_archive * arch, t_uchar * version, int full)
 }
 
 t_uchar *
-arch_archive_log (struct arch_archive * arch, t_uchar * revision)
+arch_archive_log (struct arch_archive * arch,
+                  const t_uchar * revision)
 {
   t_uchar * answer = 0;
 
@@ -303,8 +319,10 @@ arch_archive_log (struct arch_archive * arch, t_uchar * revision)
 
 
 void
-arch_revision_type (enum arch_revision_type * type, int * is_cached,
-                   struct arch_archive * arch, t_uchar * revision)
+arch_revision_type (enum arch_revision_type * type,
+                    int * is_cached,
+                    struct arch_archive * arch,
+                    const t_uchar * revision)
 {
   if (arch->vtable->revision_type (type, is_cached, arch, revision) < 0)
     {
@@ -315,7 +333,8 @@ arch_revision_type (enum arch_revision_type * type, int * is_cached,
 
 
 int
-arch_revision_exists (struct arch_archive * arch, t_uchar * revision)
+arch_revision_exists (struct arch_archive * arch,
+                      const t_uchar * revision)
 {
   enum arch_revision_type type;
   int is_cached;
@@ -324,13 +343,18 @@ arch_revision_exists (struct arch_archive * arch, t_uchar * revision)
 
 
 void
-arch_get_patch_targz (int out_fd, struct arch_archive * arch, t_uchar * revision)
+arch_get_patch_targz (int out_fd,
+                      struct arch_archive * arch,
+                      const t_uchar * revision)
 {
   arch->vtable->get_patch (out_fd, arch, revision);
 }
 
+
 void
-arch_get_patch (struct arch_archive * arch, t_uchar * revision, t_uchar * dest_dir)
+arch_get_patch (struct arch_archive * arch,
+                const t_uchar * revision,
+                const t_uchar * dest_dir)
 {
   int here_fd;
   t_uchar * dest_dir_dir = 0;
@@ -379,14 +403,20 @@ arch_get_patch (struct arch_archive * arch, t_uchar * revision, t_uchar * dest_d
   lim_free (0, patches_file_dest);
 }
 
+
 void
-arch_get_cached_revision_targz (int out_fd, struct arch_archive * arch, t_uchar * revision)
+arch_get_cached_revision_targz (int out_fd,
+                                struct arch_archive * arch,
+                                const t_uchar * revision)
 {
   arch->vtable->get_cached (out_fd, arch, revision);
 }
 
+
 void
-arch_get_cached_revision (struct arch_archive * arch, t_uchar * revision, t_uchar * dest_dir)
+arch_get_cached_revision (struct arch_archive * arch,
+                          const t_uchar * revision,
+                          const t_uchar * dest_dir)
 {
   int here_fd;
   t_uchar * dest_dir_dir = 0;
@@ -432,15 +462,20 @@ arch_get_cached_revision (struct arch_archive * arch, t_uchar * revision, t_ucha
   lim_free (0, patches_file_dest);
 }
 
+
 void
-arch_get_import_targz (int out_fd, struct arch_archive * arch, t_uchar * revision)
+arch_get_import_targz (int out_fd,
+                       struct arch_archive * arch,
+                       const t_uchar * revision)
 {
   arch->vtable->get_import (out_fd, arch, revision);
 }
 
 
 void
-arch_get_import_revision (struct arch_archive * arch, t_uchar * revision, t_uchar * dest_dir)
+arch_get_import_revision (struct arch_archive * arch,
+                          const t_uchar * revision,
+                          const t_uchar * dest_dir)
 {
   int here_fd;
   t_uchar * dest_dir_dir = 0;
@@ -488,7 +523,8 @@ arch_get_import_revision (struct arch_archive * arch, t_uchar * revision, t_ucha
 
 
 t_uchar *
-arch_get_continuation (struct arch_archive * arch, t_uchar * revision)
+arch_get_continuation (struct arch_archive * arch,
+                       const t_uchar * revision)
 {
   t_uchar * raw_data = 0;
   t_uchar * start;
@@ -513,7 +549,8 @@ arch_get_continuation (struct arch_archive * arch, t_uchar * revision)
 }
 
 t_uchar *
-arch_get_meta_info (struct arch_archive * arch, t_uchar * meta_info_name)
+arch_get_meta_info (struct arch_archive * arch,
+                    const t_uchar * meta_info_name)
 {
   t_uchar * answer = 0;
 
@@ -523,9 +560,29 @@ arch_get_meta_info (struct arch_archive * arch, t_uchar * meta_info_name)
 }
 
 
+t_uchar *
+arch_get_meta_info_trimming (struct arch_archive * arch,
+                             const t_uchar * meta_info_name)
+{
+  t_uchar * intermediate = 0;
+  t_uchar * answer = 0;
+
+  intermediate = arch_get_meta_info (arch, meta_info_name);
+  if (!intermediate)
+    return 0;
+
+  answer = str_save_trimming (0, intermediate);
+
+  lim_free (0, intermediate);
+
+  return answer;
+}
+
+
 int
 arch_make_category (t_uchar ** errstr,
-                    struct arch_archive * arch, t_uchar * category)
+                    struct arch_archive * arch,
+                    const t_uchar * category)
 {
   int answer;
 
@@ -537,7 +594,8 @@ arch_make_category (t_uchar ** errstr,
 
 int
 arch_make_branch (t_uchar ** errstr,
-                    struct arch_archive * arch, t_uchar * branch)
+                  struct arch_archive * arch,
+                  const t_uchar * branch)
 {
   int answer;
 
@@ -549,7 +607,8 @@ arch_make_branch (t_uchar ** errstr,
 
 int
 arch_make_version (t_uchar ** errstr,
-                    struct arch_archive * arch, t_uchar * version)
+                   struct arch_archive * arch,
+                   const t_uchar * version)
 {
   int answer;
 
@@ -563,12 +622,13 @@ arch_make_version (t_uchar ** errstr,
 
 
 int
-arch_archive_lock_revision (t_uchar ** errstr, struct arch_archive * a,
-                            t_uchar * version,
-                            t_uchar * prev_level,
-                            t_uchar * uid,
-                            t_uchar * txn_id,
-                            t_uchar * new_level)
+arch_archive_lock_revision (t_uchar ** errstr,
+                            struct arch_archive * a,
+                            const t_uchar * version,
+                            const t_uchar * prev_level,
+                            const t_uchar * uid,
+                            const t_uchar * txn_id,
+                            const t_uchar * new_level)
 {
   ensure_writable (a, 1);
   return a->vtable->lock_revision (errstr, a, version, prev_level, uid, txn_id, new_level);
@@ -576,12 +636,13 @@ arch_archive_lock_revision (t_uchar ** errstr, struct arch_archive * a,
 
 
 int
-arch_revision_ready (t_uchar ** errstr, struct arch_archive *a,
-                     t_uchar * version,
-                     t_uchar * prev_level,
-                     t_uchar * uid,
-                     t_uchar * txn_id,
-                     t_uchar * new_level)
+arch_revision_ready (t_uchar ** errstr,
+                     struct arch_archive *a,
+                     const t_uchar * version,
+                     const t_uchar * prev_level,
+                     const t_uchar * uid,
+                     const t_uchar * txn_id,
+                     const t_uchar * new_level)
 {
   return a->vtable->revision_ready (errstr, a, version, prev_level, uid, txn_id, new_level);
 }
@@ -589,12 +650,13 @@ arch_revision_ready (t_uchar ** errstr, struct arch_archive *a,
 
 
 int
-arch_archive_finish_revision (t_uchar ** errstr, struct arch_archive * a,
-                              t_uchar * version,
-                              t_uchar * prev_level,
-                              t_uchar * uid,
-                              t_uchar * txn_id,
-                              t_uchar * new_level)
+arch_archive_finish_revision (t_uchar ** errstr,
+                              struct arch_archive * a,
+                              const t_uchar * version,
+                              const t_uchar * prev_level,
+                              const t_uchar * uid,
+                              const t_uchar * txn_id,
+                              const t_uchar * new_level)
 {
   ensure_writable (a, 0);
   return a->vtable->finish_revision (errstr, a, version, prev_level, uid, txn_id, new_level);
@@ -605,18 +667,19 @@ arch_archive_revision_lock_state (t_uchar ** prev_level_ret,
                                   t_uchar ** uid_ret,
                                   t_uchar ** txn_id_ret,
                                   struct arch_archive * a,
-                                  t_uchar * version)
+                                  const t_uchar * version)
 {
   ensure_writable (a, 1);
   return a->vtable->lock_state (prev_level_ret, uid_ret, txn_id_ret, a, version);
 }
 
 int
-arch_archive_break_revision_lock (t_uchar ** errstr, struct arch_archive * a,
-                                  t_uchar * version,
-                                  t_uchar * prev_level,
-                                  t_uchar * uid,
-                                  t_uchar * txn_id)
+arch_archive_break_revision_lock (t_uchar ** errstr,
+                                  struct arch_archive * a,
+                                  const t_uchar * version,
+                                  const t_uchar * prev_level,
+                                  const t_uchar * uid,
+                                  const t_uchar * txn_id)
 {
   ensure_writable (a, 1);
   return a->vtable->break_revision_lock (errstr, a, version, prev_level, uid, txn_id);
@@ -626,12 +689,13 @@ arch_archive_break_revision_lock (t_uchar ** errstr, struct arch_archive * a,
 
 
 int
-arch_archive_put_log (t_uchar ** errstr, struct arch_archive * a,
-                      t_uchar * version,
-                      t_uchar * prev_level,
-                      t_uchar * uid,
-                      t_uchar * txn_id,
-                      t_uchar * log_text)
+arch_archive_put_log (t_uchar ** errstr,
+                      struct arch_archive * a,
+                      const t_uchar * version,
+                      const t_uchar * prev_level,
+                      const t_uchar * uid,
+                      const t_uchar * txn_id,
+                      const t_uchar * log_text)
 {
   ensure_writable (a, 0);
   return a->vtable->put_log (errstr, a, version, prev_level, uid, txn_id, log_text);
@@ -640,12 +704,13 @@ arch_archive_put_log (t_uchar ** errstr, struct arch_archive * a,
 
 
 int
-arch_archive_put_continuation (t_uchar ** errstr, struct arch_archive * a,
-                               t_uchar * version,
-                               t_uchar * prev_level,
-                               t_uchar * uid,
-                               t_uchar * txn_id,
-                               t_uchar * continuation)
+arch_archive_put_continuation (t_uchar ** errstr,
+                               struct arch_archive * a,
+                               const t_uchar * version,
+                               const t_uchar * prev_level,
+                               const t_uchar * uid,
+                               const t_uchar * txn_id,
+                               const t_uchar * continuation)
 {
   ensure_writable (a, 0);
   return a->vtable->put_continuation (errstr, a, version, prev_level, uid, txn_id, continuation);
@@ -653,12 +718,13 @@ arch_archive_put_continuation (t_uchar ** errstr, struct arch_archive * a,
 
 
 int
-arch_archive_put_changeset_targz (t_uchar ** errstr, struct arch_archive * a,
-                                  t_uchar * version,
-                                  t_uchar * prev_level,
-                                  t_uchar * uid,
-                                  t_uchar * txn_id,
-                                  t_uchar * level,
+arch_archive_put_changeset_targz (t_uchar ** errstr,
+                                  struct arch_archive * a,
+                                  const t_uchar * version,
+                                  const t_uchar * prev_level,
+                                  const t_uchar * uid,
+                                  const t_uchar * txn_id,
+                                  const t_uchar * level,
                                   int in_fd)
 {
   int answer;
@@ -670,13 +736,14 @@ arch_archive_put_changeset_targz (t_uchar ** errstr, struct arch_archive * a,
 
 
 int
-arch_archive_put_changeset (t_uchar ** errstr, struct arch_archive * a,
-                            t_uchar * version,
-                            t_uchar * prev_level,
-                            t_uchar * uid,
-                            t_uchar * txn_id,
-                            t_uchar * level,
-                            t_uchar * dir)
+arch_archive_put_changeset (t_uchar ** errstr,
+                            struct arch_archive * a,
+                            const t_uchar * version,
+                            const t_uchar * prev_level,
+                            const t_uchar * uid,
+                            const t_uchar * txn_id,
+                            const t_uchar * level,
+                            const t_uchar * dir)
 {
   t_uchar * dir_tail = 0;
   t_uchar * desired_name = 0;
@@ -712,12 +779,13 @@ arch_archive_put_changeset (t_uchar ** errstr, struct arch_archive * a,
 
 
 int
-arch_archive_put_import_targz (t_uchar ** errstr, struct arch_archive * a,
-                               t_uchar * version,
-                               t_uchar * prev_level,
-                               t_uchar * uid,
-                               t_uchar * txn_id,
-                               t_uchar * level,
+arch_archive_put_import_targz (t_uchar ** errstr,
+                               struct arch_archive * a,
+                               const t_uchar * version,
+                               const t_uchar * prev_level,
+                               const t_uchar * uid,
+                               const t_uchar * txn_id,
+                               const t_uchar * level,
                                int in_fd)
 {
   int answer;
@@ -729,13 +797,14 @@ arch_archive_put_import_targz (t_uchar ** errstr, struct arch_archive * a,
 
 
 int
-arch_archive_put_import (t_uchar ** errstr, struct arch_archive * a,
-                         t_uchar * version,
-                         t_uchar * prev_level,
-                         t_uchar * uid,
-                         t_uchar * txn_id,
-                         t_uchar * level,
-                         t_uchar * dir)
+arch_archive_put_import (t_uchar ** errstr,
+                         struct arch_archive * a,
+                         const t_uchar * version,
+                         const t_uchar * prev_level,
+                         const t_uchar * uid,
+                         const t_uchar * txn_id,
+                         const t_uchar * level,
+                         const t_uchar * dir)
 {
   t_uchar * dir_tail = 0;
   t_uchar * desired_name = 0;
@@ -770,7 +839,10 @@ arch_archive_put_import (t_uchar ** errstr, struct arch_archive * a,
 }
 
 int
-arch_archive_put_cached_targz (t_uchar ** errstr, struct arch_archive * a, t_uchar * revision, int in_fd)
+arch_archive_put_cached_targz (t_uchar ** errstr,
+                               struct arch_archive * a,
+                               const t_uchar * revision,
+                               int in_fd)
 {
   int answer;
 
@@ -780,7 +852,10 @@ arch_archive_put_cached_targz (t_uchar ** errstr, struct arch_archive * a, t_uch
 }
 
 int
-arch_archive_put_cached (t_uchar ** errstr, struct arch_archive * a, t_uchar * revision, t_uchar * dir)
+arch_archive_put_cached (t_uchar ** errstr,
+                         struct arch_archive * a,
+                         const t_uchar * revision,
+                         const t_uchar * dir)
 {
   t_uchar * dir_tail = 0;
   t_uchar * tar_file_path = 0;
@@ -813,7 +888,9 @@ arch_archive_put_cached (t_uchar ** errstr, struct arch_archive * a, t_uchar * r
 
 
 int
-arch_archive_delete_cached (t_uchar ** errstr, struct arch_archive * a, t_uchar * revision)
+arch_archive_delete_cached (t_uchar ** errstr,
+                            struct arch_archive * a,
+                            const t_uchar * revision)
 {
   ensure_writable (a, 1);
   return a->vtable->delete_cached (errstr, a, revision);
@@ -859,7 +936,8 @@ arch_generate_txn_id (void)
 
 
 t_uchar *
-arch_previous_revision (struct arch_archive * arch, t_uchar * revision)
+arch_previous_revision (struct arch_archive * arch,
+                        const t_uchar * revision)
 {
   t_uchar * version = 0;
   t_uchar * patch_level = 0;
@@ -897,19 +975,19 @@ arch_previous_revision (struct arch_archive * arch, t_uchar * revision)
         }
       else
         {
-          rel_table revisions = 0;
+          rel_table revisions  = rel_table_nil;
           int x;
 
           revisions = arch_archive_revisions (arch, version, 0);
 
-          if (!revisions)
+          if (!rel_n_records (revisions))
             {
               safe_printfmt (2, "version has no revisions (%s/%s)\n", arch->name, version);
               exit (2);
             }
 
           for (x = 0; x < rel_n_records (revisions); ++x)
-            if (revisions[x][0][0] == 'v')
+            if (rel_peek_str (revisions, x, 0)[0] == 'v')
               break;
 
           invariant (x);
@@ -944,7 +1022,8 @@ arch_previous_revision (struct arch_archive * arch, t_uchar * revision)
 
 
 t_uchar *
-arch_ancestor_revision (struct arch_archive * arch, t_uchar * revision)
+arch_ancestor_revision (struct arch_archive * arch,
+                        const t_uchar * revision)
 {
   enum arch_revision_type type;
 
@@ -983,7 +1062,8 @@ arch_ancestor_revision (struct arch_archive * arch, t_uchar * revision)
 
 
 t_uchar *
-archive_tmp_file_name (t_uchar * dir, t_uchar * basename)
+archive_tmp_file_name (const t_uchar * dir,
+                       const t_uchar * basename)
 {
   t_uchar * my_uid = 0;
   t_uchar * tmp_name = 0;
@@ -1085,7 +1165,7 @@ wait_for_tar (int pid)
 
 
 t_uchar *
-make_tmp_tar_archive (t_uchar * dir)
+make_tmp_tar_archive (const t_uchar * dir)
 {
   t_uchar * dir_dir = 0;
   t_uchar * dir_tail = 0;
@@ -1187,7 +1267,8 @@ make_tmp_tar_archive (t_uchar * dir)
 
 
 static void
-ensure_writable (struct arch_archive * arch, int mirror_ok)
+ensure_writable (struct arch_archive * arch,
+                 int mirror_ok)
 {
   if (arch->access != arch_archive_writable)
     {
@@ -1218,8 +1299,10 @@ ensure_writable (struct arch_archive * arch, int mirror_ok)
     }
 }
 
+
 int
-arch_get_meta_int_info(struct arch_archive * arch, t_uchar * key)
+arch_get_meta_int_info(struct arch_archive * arch,
+                       const t_uchar * key)
 {
   t_uchar * key_existence = arch_get_meta_info (arch, key);
   int result  = !!key_existence;
