@@ -24,10 +24,10 @@
 
 
 /* __STDC__ prototypes for static functions */
-static void changeset_inv_callback (t_uchar * path,
+static void changeset_inv_callback (const t_uchar * path,
                                     struct stat * stat_buf,
                                     enum arch_inventory_category category,
-                                    t_uchar * id,
+                                    const t_uchar * id,
                                     int has_source_name,
                                     void * closure,
                                     int escape_classes);
@@ -36,7 +36,8 @@ static void changeset_inv_callback (t_uchar * path,
 
 void
 arch_changeset_inventory (struct arch_changeset_inventory * inv_out,
-                          t_uchar * tree_root, t_uchar * path,
+                          const t_uchar * tree_root,
+                          const t_uchar * path,
                           enum arch_id_tagging_method method,
                           enum arch_inventory_category untagged_source_category,
                           int escape_classes)
@@ -80,17 +81,16 @@ arch_changeset_inventory (struct arch_changeset_inventory * inv_out,
 void
 arch_free_changeset_inventory_data (struct arch_changeset_inventory * i)
 {
-  rel_free_table (i->dirs);     i->dirs = 0;
-  rel_free_table (i->files);    i->files = 0;
+  rel_free_table (i->dirs);     i->dirs = rel_table_nil;
+  rel_free_table (i->files);    i->files = rel_table_nil;
 }
 
 
-
 rel_table
-arch_read_changeset_index (t_uchar * path)
+arch_read_changeset_index (const t_uchar * path)
 {
   int in_fd;
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
   t_uchar * line;
   long len;
 
@@ -165,7 +165,7 @@ arch_read_changeset_index (t_uchar * path)
           exit (2);
         }
 
-      rel_add_records (&answer, rel_make_record (loc, id, 0), 0);
+      rel_add_records (&answer, rel_make_record_2_taking (rel_make_field_str (loc), rel_make_field_str (id)), 0);
       lim_free (0, loc);
       lim_free (0, id);
     }
@@ -176,17 +176,17 @@ arch_read_changeset_index (t_uchar * path)
 
 
 rel_table
-arch_read_changeset_dir_metadata (t_uchar * path)
+arch_read_changeset_dir_metadata (const t_uchar * path)
 {
   int errn;
   int in_fd;
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
 
   in_fd = vu_open (&errn, path, O_RDONLY, 0);
   if (in_fd < 0)
     {
       if (errn == ENOENT)
-        return 0;
+        return rel_table_nil;
       else
         {
           safe_printfmt (2, "arch_read_changeset_dir_metadata: unable to open file (%s)\n", path);
@@ -266,7 +266,7 @@ arch_read_changeset_dir_metadata (t_uchar * path)
           exit (2);
         }
 
-      rel_add_records (&answer, rel_make_record (loc, perms, 0), 0);
+      rel_add_records (&answer, rel_make_record_2_taking (rel_make_field_str (loc), rel_make_field_str (perms)), 0);
 
       lim_free (0, perms);
       lim_free (0, loc);
@@ -281,7 +281,7 @@ arch_read_changeset_dir_metadata (t_uchar * path)
 
 
 mode_t
-arch_read_permissions_patch (t_uchar * file)
+arch_read_permissions_patch (const t_uchar * file)
 {
   int errn;
   t_uchar * line = 0;
@@ -323,10 +323,10 @@ arch_read_permissions_patch (t_uchar * file)
 
 
 static void
-changeset_inv_callback (t_uchar * path,
+changeset_inv_callback (const t_uchar * path,
                         struct stat * stat_buf,
                         enum arch_inventory_category category,
-                        t_uchar * id,
+                        const t_uchar * id,
                         int has_source_name,
                         void * closure,
                         int escape_classes)
@@ -355,9 +355,9 @@ changeset_inv_callback (t_uchar * path,
 
 
   if (S_ISDIR (stat_buf->st_mode))
-    rel_add_records (&index->dirs, rel_make_record (path, id, 0), 0);
+    rel_add_records (&index->dirs, rel_make_record_2_taking (rel_make_field_str (path), rel_make_field_str (id)), 0);
   else
-    rel_add_records (&index->files, rel_make_record (path, id, 0), 0);
+    rel_add_records (&index->files, rel_make_record_2_taking (rel_make_field_str (path), rel_make_field_str (id)), 0);
 }
 
 
