@@ -161,7 +161,7 @@ arch_cmd_merges (t_uchar * program_name, int argc, char * argv[])
     t_uchar * from_archive = 0;
     t_uchar * into_spec = 0;
     t_uchar * from_spec = 0;
-    rel_table merge_points = 0;
+    rel_table merge_points = rel_table_nil;
 
     here_fd = safe_open (".", O_RDONLY, 0);
 
@@ -240,9 +240,9 @@ arch_cmd_merges (t_uchar * program_name, int argc, char * argv[])
           {
             t_uchar * level = 0;
 
-            level = arch_parse_package_name (arch_ret_patch_level, 0, merge_points[x][1]);
+            level = arch_parse_package_name (arch_ret_patch_level, 0, rel_peek_str (merge_points, x, 1));
 
-            safe_printfmt (1, "%s\t%s\n", merge_points[x][0], level);
+            safe_printfmt (1, "%s\t%s\n", rel_peek_str (merge_points, x, 0), level);
 
             lim_free (0, level);
           }
