@@ -35,7 +35,9 @@
 
 
 /* __STDC__ prototypes for static functions */
-static void library_txn_callback (void * vfp, char * fmt, va_list ap);
+static void library_txn_callback (void * vfp,
+                                  const char * fmt,
+                                  va_list ap);
 
 
 
@@ -84,13 +86,13 @@ static void library_txn_callback (void * vfp, char * fmt, va_list ap);
  * an error message is printed and the process exitted with status 2.
  */
 t_uchar *
-arch_library_add_choice (t_uchar * archive,
-                         t_uchar * revision,
-                         t_uchar * opt_library,
-                         t_uchar * opt_same_dev_path)
+arch_library_add_choice (const t_uchar * archive,
+                         const t_uchar * revision,
+                         const t_uchar * opt_library,
+                         const t_uchar * opt_same_dev_path)
 {
-  rel_table full_user_path = 0;
-  rel_table search_path = 0;
+  rel_table full_user_path = rel_table_nil;
+  rel_table search_path = rel_table_nil;
   t_uchar * target_library = 0;
 
 
@@ -125,7 +127,7 @@ arch_library_add_choice (t_uchar * archive,
 
       for (x = 0; x < rel_n_records (full_user_path); ++x)
         {
-          if (names_same_inode (opt_library, full_user_path[x][0]))
+          if (names_same_inode (opt_library, rel_peek_str (full_user_path, x, 0)))
             break;
         }
 
@@ -135,13 +137,13 @@ arch_library_add_choice (t_uchar * archive,
           exit (2);
         }
 
-      rel_add_records (&search_path, rel_make_record (full_user_path[x][0], 0), 0);
+      rel_add_records (&search_path, rel_singleton_record_taking (rel_get_field (full_user_path, x, 0)), 0);
     }
 
   else
     search_path=make_search_path(full_user_path, opt_same_dev_path, 0);
 
-  if (!search_path)
+  if (!rel_n_records (search_path))
     {
       if (opt_same_dev_path)
 	safe_printfmt (2, "tla: no library on the same device as dir\n  dir %s\n", opt_same_dev_path);
@@ -183,8 +185,7 @@ arch_library_add_choice (t_uchar * archive,
     /* Not there -- we must decide where to add it */
     else
       {
-	t_uchar *best_guess = arch_find_best_library(archive, search_path, revision);
-	target_library = str_save (0, best_guess);      
+	target_library = arch_find_best_library (archive, search_path, revision);
       }
   }
 
@@ -194,31 +195,28 @@ arch_library_add_choice (t_uchar * archive,
   return target_library;
 }
 
-/**
+/*
  * This function is similar to the above, but it may return NULL
  *
  * It finds the best greedy library to add a given revision to
  * If opt_same_dev_path has a value, matches are restricted to libraries on the
  * device containing this path
  */
-extern t_uchar * arch_library_greedy_add_choice(t_uchar * archive, 
-                                                t_uchar * revision, 
-						t_uchar * opt_same_dev_path,
-						int require_greedy)
-
+t_uchar *
+arch_library_greedy_add_choice (const t_uchar * archive, 
+                                const t_uchar * revision, 
+                                const t_uchar * opt_same_dev_path,
+                                int require_greedy)
 {
-  rel_table full_user_path = 0;
-  rel_table search_path = 0;
+  rel_table full_user_path = rel_table_nil;
+  rel_table search_path = rel_table_nil;
 
   t_uchar * target_library = 0;  
-  t_uchar * best_guess = 0;
 
   full_user_path = arch_my_library_path (arch_library_path_add_order);
   search_path = make_search_path (full_user_path, opt_same_dev_path, require_greedy);
 
-  best_guess = arch_find_best_library (archive, search_path, revision);
-  if (best_guess)
-    target_library=str_save(0, best_guess);
+  target_library = arch_find_best_library (archive, search_path, revision);
   
   rel_free_table (full_user_path);
   rel_free_table (search_path);
@@ -232,19 +230,22 @@ extern t_uchar * arch_library_greedy_add_choice(t_uchar * archive,
  * if require_greedy is true, candidates must be greedy
  * returns 0 if there are no matching candidates
  */
-rel_table make_search_path(rel_table full_user_path, t_uchar *opt_same_dev_path, int require_greedy)
+rel_table
+make_search_path (rel_table full_user_path,
+                  const t_uchar * opt_same_dev_path,
+                  int require_greedy)
 {
-  rel_table search_path = 0;
+  rel_table search_path = rel_table_nil;
   int x;
 
   for (x = 0; x < rel_n_records (full_user_path); ++x)
     {
-      if (opt_same_dev_path && !on_same_device (opt_same_dev_path, full_user_path[x][0]))
+      if (opt_same_dev_path && !on_same_device (opt_same_dev_path, rel_peek_str (full_user_path, x, 0)))
 	continue;
 
-      if (require_greedy && !arch_library_is_greedy(full_user_path[x][0]))
+      if (require_greedy && !arch_library_is_greedy(rel_peek_str (full_user_path, x, 0)))
 	continue;
-      rel_add_records (&search_path, rel_make_record (full_user_path[x][0], 0), 0);
+      rel_add_records (&search_path, rel_singleton_record_taking (rel_get_field (full_user_path, x, 0)), 0);
     }
 
   return search_path;
@@ -256,58 +257,67 @@ rel_table make_search_path(rel_table full_user_path, t_uchar *opt_same_dev_path,
  * branch, category, or archive) -- defaulting to the first in the 
  * path.
  */
-t_uchar * arch_find_best_library(t_uchar * archive, 
-				  rel_table search_path,
-				  t_uchar * revision)
+t_uchar *
+arch_find_best_library(const t_uchar * archive, 
+                       rel_table search_path,
+                       const t_uchar * revision)
 {
-  int guess_priority;
-  int x;
-  t_uchar * best_guess = 0;
+  const t_uchar * best_guess;
 
-  t_uchar * category = 0;
-  t_uchar * branch = 0;
-  t_uchar * version = 0;
-  
   if (rel_n_records (search_path) == 0)
     return 0;
+
   if (rel_n_records (search_path) == 1)
-    return search_path[0][0];
-  
-  category = arch_parse_package_name (arch_ret_category, 0, revision);
-  branch = arch_parse_package_name (arch_ret_package, 0, revision);
-  version = arch_parse_package_name (arch_ret_package_version, 0, revision);
-
-  best_guess = search_path[0][0];
-  guess_priority = 0;
-
-  for (x = 0; x < rel_n_records (search_path); ++x)
     {
-      if (arch_library_has_version (search_path[x][0], archive, version))
-	{
-	  best_guess = search_path[x][0];
-	  break;
-	}
-      else if (arch_library_has_branch (search_path[x][0], archive, branch))
-	{
-	  best_guess = search_path[x][0];
-	  guess_priority = 3;
-	}
-      else if ((guess_priority < 2) && arch_library_has_category (search_path[x][0], archive, category))
-	{
-	  best_guess = search_path[x][0];
-	  guess_priority = 2;
-	}
-      else if ((guess_priority < 1) && arch_library_has_archive (search_path[x][0], archive))
-	{
-	  best_guess = search_path[x][0];
-	  guess_priority = 1;
-	}
+      best_guess = rel_peek_str (search_path, 0, 0);
     }
-  lim_free (0, category);
-  lim_free (0, branch);
-  lim_free (0, version);
+  else
+    {
+      t_uchar * category = 0;
+      t_uchar * branch = 0;
+      t_uchar * version = 0;
+      int guess_priority;
+      int x;
 
-  return best_guess;
+      category = arch_parse_package_name (arch_ret_category, 0, revision);
+      branch = arch_parse_package_name (arch_ret_package, 0, revision);
+      version = arch_parse_package_name (arch_ret_package_version, 0, revision);
+      
+      best_guess = rel_peek_str (search_path, 0, 0);
+      guess_priority = 0;
+      
+      for (x = 0; x < rel_n_records (search_path); ++x)
+        {
+          if (arch_library_has_version (rel_peek_str (search_path, x, 0), archive, version))
+            {
+              best_guess = rel_peek_str (search_path, x, 0);
+              break;
+            }
+          else if (arch_library_has_branch (rel_peek_str (search_path, x, 0), archive, branch))
+            {
+              best_guess = rel_peek_str (search_path, x, 0);
+              guess_priority = 3;
+            }
+          else if ((guess_priority < 2) && arch_library_has_category (rel_peek_str (search_path, x, 0), archive, category))
+            {
+              best_guess = rel_peek_str (search_path, x, 0);
+              guess_priority = 2;
+            }
+          else if ((guess_priority < 1) && arch_library_has_archive (rel_peek_str (search_path, x, 0), archive))
+            {
+              best_guess = rel_peek_str (search_path, x, 0);
+              guess_priority = 1;
+            }
+        }
+      lim_free (0, category);
+      lim_free (0, branch);
+      lim_free (0, version);
+    }
+      
+  if (!best_guess)
+    return 0;
+  else
+    return str_save (0, best_guess);
 }
 
 
@@ -315,9 +325,9 @@ void
 arch_library_add (int chatter_fd,
                   int no_patch_noise,
                   struct arch_archive * arch,
-                  t_uchar * revision,
-                  t_uchar * opt_library,
-                  t_uchar * opt_same_dev_path,
+                  const t_uchar * revision,
+                  const t_uchar * opt_library,
+                  const t_uchar * opt_same_dev_path,
                   int sparse,
                   int escape_classes)
 {
@@ -358,7 +368,7 @@ arch_library_add (int chatter_fd,
     {
       ancestor_archive = arch_parse_package_name (arch_ret_archive, 0, fq_ancestor_rev);
       ancestor_revision = arch_parse_package_name (arch_ret_non_archive, 0, fq_ancestor_rev);
-      library_ancestor = arch_library_find(0, ancestor_archive, ancestor_revision, 1);
+      library_ancestor = arch_library_find (rel_table_nil, ancestor_archive, ancestor_revision, 1);
     }
 
   dest_dir_dir = file_name_directory_file (0, dest_directory);
@@ -412,7 +422,7 @@ arch_library_add (int chatter_fd,
        */
       arch_chatter (chatter_fd, "* recursively adding %s/%s to library\n", ancestor_archive, ancestor_revision);
       arch_library_add (chatter_fd, no_patch_noise, arch, ancestor_revision, opt_library, opt_same_dev_path, 0, escape_classes);
-      library_ancestor = arch_library_find (0, ancestor_archive, ancestor_revision, 1);
+      library_ancestor = arch_library_find (rel_table_nil, ancestor_archive, ancestor_revision, 1);
       build_link_tree (library_ancestor, tmp_dir);
     }
   else
@@ -432,11 +442,11 @@ arch_library_add (int chatter_fd,
       int replay_status = 1;
       /* list of patches to apply to revision in library to achieve new revision
        */
-      rel_table revisions = 0;
+      rel_table revisions = rel_table_nil;
 
       /* this is the first patch missing
        */
-      rel_add_records (&revisions,  rel_make_record (revision, 0), 0);
+      rel_add_records (&revisions,  rel_singleton_record_taking (rel_make_field_str (revision)), 0);
 
       arch_chatter (chatter_fd, "* searching ancestor revision in library in archive %s\n", ancestor_archive);
 
@@ -450,7 +460,7 @@ arch_library_add (int chatter_fd,
           /* add revision to list
            */
 
-          rel_add_records (&revisions,  rel_make_record (anc_rev, 0), 0);
+          rel_add_records (&revisions,  rel_singleton_record_taking (rel_make_field_str (anc_rev)), 0);
 
           /* look in the library for the immediate ancestor
            */
@@ -463,7 +473,7 @@ arch_library_add (int chatter_fd,
 
               anc_arch = arch_parse_package_name (arch_ret_archive, 0, fq_anc_rev);
               anc_rev = arch_parse_package_name (arch_ret_non_archive, 0, fq_anc_rev);
-              anc_lib = arch_library_find (0, anc_arch, anc_rev, 1);
+              anc_lib = arch_library_find (rel_table_nil, anc_arch, anc_rev, 1);
             }
 
           if (!fq_anc_rev || str_cmp (anc_arch, arch->name))
@@ -568,7 +578,7 @@ arch_library_add (int chatter_fd,
 
   {
     t_uchar * version = 0;
-    rel_table index = 0;
+    rel_table index = rel_table_nil;
     t_uchar * index_by_name_path = 0;
     t_uchar * index_path = 0;
     int index_by_name_fd = -1;
@@ -619,7 +629,9 @@ arch_library_add (int chatter_fd,
 
 
 static void
-library_txn_callback (void * vfp, char * fmt, va_list ap)
+library_txn_callback (void * vfp,
+                      const char * fmt,
+                      va_list ap)
 {
   int fd;
 
