@@ -22,7 +22,6 @@
 #include "tla/libfsutils/ensure-dir.h"
 #include "tla/libfsutils/rmrf.h"
 #include "tla/libfsutils/copy-file.h"
-#include "tla/libawk/trim.h"
 #include "tla/libarch/arch.h"
 #include "tla/libarch/namespace.h"
 #include "tla/libarch/merge-points.h"
@@ -32,7 +31,9 @@
 
 
 t_uchar *
-arch_make_log_file (t_uchar * tree_root, t_uchar * archive, t_uchar * version)
+arch_make_log_file (const t_uchar * tree_root,
+                    const t_uchar * archive,
+                    const t_uchar * version)
 {
   t_uchar * log_name = 0;
   t_uchar * log_path = 0;
@@ -51,7 +52,9 @@ arch_make_log_file (t_uchar * tree_root, t_uchar * archive, t_uchar * version)
 
 
 t_uchar *
-arch_make_log (t_uchar * tree_root, t_uchar * archive, t_uchar * version)
+arch_make_log (const t_uchar * tree_root,
+               const t_uchar * archive,
+               const t_uchar * version)
 {
   int errn;
   t_uchar * log_path = 0;
@@ -75,14 +78,17 @@ arch_make_log (t_uchar * tree_root, t_uchar * archive, t_uchar * version)
   return log_path;
 }
 
+
 t_uchar *
-arch_auto_log_message (t_uchar * tree_root, t_uchar * archive, t_uchar * version, t_uchar * summary, t_uchar * body)
+arch_auto_log_message (const t_uchar * tree_root,
+                       const t_uchar * archive,
+                       const t_uchar * version,
+                       const t_uchar * summary,
+                       const t_uchar * body)
 {
-  t_uchar * cooked_summary = 0;
   t_uchar * merge_summary = 0;
   t_uchar * answer = 0;
 
-  cooked_summary = trim_surrounding_ws (str_save (0, summary));
   merge_summary = arch_log_for_merge (tree_root, archive, version, 0);
 
   answer = str_alloc_cat_many (0,
@@ -94,16 +100,18 @@ arch_auto_log_message (t_uchar * tree_root, t_uchar * archive, t_uchar * version
                                merge_summary,
                                str_end);
 
-  lim_free (0, cooked_summary);
   lim_free (0, merge_summary);
   return answer;
 }
 
 
 t_uchar *
-arch_log_for_merge (t_uchar * tree_root, t_uchar * archive, t_uchar * version, int reverse)
+arch_log_for_merge (const t_uchar * tree_root,
+                    const t_uchar * archive,
+                    const t_uchar * version,
+                    int reverse)
 {
-  rel_table new_logs = 0;
+  rel_table new_logs = rel_table_nil;
   int string_fd;
 
   new_logs = arch_new_in_version (tree_root, archive, version);
@@ -113,7 +121,7 @@ arch_log_for_merge (t_uchar * tree_root, t_uchar * archive, t_uchar * version, i
 
   string_fd = make_output_to_string_fd ();
 
-  if (new_logs)
+  if (rel_n_records (new_logs))
     {
       int x;
 
@@ -126,10 +134,10 @@ arch_log_for_merge (t_uchar * tree_root, t_uchar * archive, t_uchar * version, i
           t_uchar * log_file = 0;
           t_uchar * log = 0;
           assoc_table headers = 0;
-          t_uchar * summary;
+          const t_uchar * summary;
 
-          archive = arch_parse_package_name (arch_ret_archive, 0, new_logs[x][0]);
-          revision = arch_parse_package_name (arch_ret_non_archive, 0, new_logs[x][0]);
+          archive = arch_parse_package_name (arch_ret_archive, 0, rel_peek_str (new_logs, x, 0));
+          revision = arch_parse_package_name (arch_ret_non_archive, 0, rel_peek_str (new_logs, x, 0));
 
           invariant (!!archive && !!revision);
 
@@ -137,9 +145,9 @@ arch_log_for_merge (t_uchar * tree_root, t_uchar * archive, t_uchar * version, i
           log = file_contents (log_file);
 
           arch_parse_log (0, &headers, 0, log);
-          summary = assoc_ref (headers, "summary");
+          summary = assoc_get_str_taking (headers, rel_make_field_str ("summary"));
 
-          safe_printfmt (string_fd, " * %s\n", new_logs[x][0]);
+          safe_printfmt (string_fd, " * %s\n", rel_peek_str (new_logs, x, 0));
           safe_printfmt (string_fd, "   %s\n\n", (summary ? summary : (t_uchar *)"<no summary provided>"));
 
           lim_free (0, archive);
@@ -156,7 +164,9 @@ arch_log_for_merge (t_uchar * tree_root, t_uchar * archive, t_uchar * version, i
 
 
 t_uchar *
-arch_log_dir (t_uchar * tree_root, t_uchar * archive, t_uchar * version)
+arch_log_dir (const t_uchar * tree_root,
+              const t_uchar * archive,
+              const t_uchar * version)
 {
   t_uchar * category;
   t_uchar * package;
@@ -198,7 +208,9 @@ arch_log_dir (t_uchar * tree_root, t_uchar * archive, t_uchar * version)
 
 
 t_uchar *
-arch_log_file (t_uchar * tree_root, t_uchar * archive, t_uchar * revision)
+arch_log_file (const t_uchar * tree_root,
+               const t_uchar * archive,
+               const t_uchar * revision)
 {
   t_uchar * version;
   t_uchar * dir;
@@ -217,8 +229,11 @@ arch_log_file (t_uchar * tree_root, t_uchar * archive, t_uchar * revision)
   return file;
 }
 
+
 t_uchar *
-arch_log_contents (t_uchar * tree_root, t_uchar * archive, t_uchar * revision)
+arch_log_contents (const t_uchar * tree_root,
+                   const t_uchar * archive,
+                   const t_uchar * revision)
 {
   t_uchar * file = 0;
   t_uchar * answer = 0;
@@ -234,9 +249,10 @@ arch_log_contents (t_uchar * tree_root, t_uchar * archive, t_uchar * revision)
 }
 
 
-
 int
-arch_tree_has_log (t_uchar * tree_root, t_uchar * archive, t_uchar * revision)
+arch_tree_has_log (const t_uchar * tree_root,
+                   const t_uchar * archive,
+                   const t_uchar * revision)
 {
   t_uchar * file = 0;
   int answer;
@@ -250,9 +266,11 @@ arch_tree_has_log (t_uchar * tree_root, t_uchar * archive, t_uchar * revision)
   return answer;
 }
 
+
 void
-arch_add_log_version (t_uchar * tree_root, t_uchar * archive,
-                      t_uchar * version)
+arch_add_log_version (const t_uchar * tree_root,
+                      const t_uchar * archive,
+                      const t_uchar * version)
 {
   t_uchar * log_dir;
 
@@ -264,8 +282,9 @@ arch_add_log_version (t_uchar * tree_root, t_uchar * archive,
 
 
 void
-arch_remove_log_version (t_uchar * tree_root, t_uchar * archive,
-                         t_uchar * version)
+arch_remove_log_version (const t_uchar * tree_root,
+                         const t_uchar * archive,
+                         const t_uchar * version)
 {
   t_uchar * log_dir;
 
@@ -275,8 +294,11 @@ arch_remove_log_version (t_uchar * tree_root, t_uchar * archive,
   lim_free (0, log_dir);
 }
 
+
 int
-arch_has_patch_log (t_uchar * tree_root, t_uchar * archive, t_uchar * version)
+arch_has_patch_log (const t_uchar * tree_root,
+                    const t_uchar * archive,
+                    const t_uchar * version)
 {
   t_uchar * dir = 0;
   int answer = 0;
@@ -291,14 +313,16 @@ arch_has_patch_log (t_uchar * tree_root, t_uchar * archive, t_uchar * version)
 }
 
 
-
 rel_table
-arch_log_versions (t_uchar * tree_root,
-                   t_uchar * only_archive,
-                   t_uchar * only_category,
-                   t_uchar * only_branch,
-                   t_uchar * only_version)
+arch_log_versions (const t_uchar * tree_root,
+                   const t_uchar * only_archive,
+                   const t_uchar * only_category_param,
+                   const t_uchar * only_branch_param,
+                   const t_uchar * only_version_param)
 {
+  t_uchar * only_category = 0;
+  t_uchar * only_branch = 0;
+  t_uchar * only_version = 0;
   t_uchar * logs_root;
   int lim;
   int x;
@@ -317,22 +341,28 @@ arch_log_versions (t_uchar * tree_root,
   logs_root = arch_tree_ctl_dir (tree_root);
 
 
-  if (only_version)
-    only_version = str_save (0, only_version);
+  if (only_version_param)
+    only_version = str_save (0, only_version_param);
+  else
+    only_version = 0;
 
-  if (only_version && !only_branch)
+  if (only_version && !only_branch_param)
     {
       only_branch = arch_parse_package_name (arch_ret_package, 0, only_version);
     }
-  else if (only_branch)
-    only_branch = str_save (0, only_branch);
+  else if (only_branch_param)
+    only_branch = str_save (0, only_branch_param);
+  else
+    only_branch = 0;
 
-  if (only_branch && !only_category)
+  if (only_branch && !only_category_param)
     {
       only_category = arch_parse_package_name (arch_ret_category, 0, only_branch);
     }
-  else if (only_category)
-    only_category = str_save (0, only_category);
+  else if (only_category_param)
+    only_category = str_save (0, only_category_param);
+  else
+    only_category = 0;
 
 
 
@@ -342,15 +372,15 @@ arch_log_versions (t_uchar * tree_root,
 
   dir_listing = directory_files (logs_root);
 
-  categories = 0;
+  categories = rel_table_nil;
   lim = rel_n_records (dir_listing);
   for (x = 0; x < lim; ++x)
     {
-      if (only_category && str_cmp (dir_listing[x][0], only_category))
+      if (only_category && str_cmp (rel_peek_str (dir_listing, x, 0), only_category))
         continue;
 
-      if (arch_valid_package_name (dir_listing[x][0], arch_no_archive, arch_req_category, 0))
-        rel_add_records (&categories, rel_copy_record (dir_listing[x]), 0);
+      if (arch_valid_package_name (rel_peek_str (dir_listing, x, 0), arch_no_archive, arch_req_category, 0))
+        rel_add_records (&categories, rel_copy_record (rel_peek_record (dir_listing, x)), 0);
     }
   rel_free_table (dir_listing);
 
@@ -358,24 +388,26 @@ arch_log_versions (t_uchar * tree_root,
    * For each category, get a list of each branch for it in the log
    */
 
-  branches = 0;
+  branches = rel_table_nil;
   c_lim = rel_n_records (categories);
   for (c = 0; c < c_lim; ++c)
     {
-      t_uchar * category;
-      t_uchar * cat_dir;
+      const t_uchar * category;
+      t_uchar * cat_dir = 0;
 
-      category = categories[c][0];
+      category = rel_peek_str (categories, c, 0);
       cat_dir = file_name_in_vicinity (0, logs_root, category);
 
       dir_listing = directory_files (cat_dir);
       lim = rel_n_records (dir_listing);
       for (x = 0; x < lim; ++x)
         {
-          t_uchar * maybe_branch;
-          t_uchar * branch_cat;
+          rel_field maybe_branch_field;
+          const t_uchar * maybe_branch;
+          t_uchar * branch_cat = 0;
 
-          maybe_branch = dir_listing[x][0];
+          maybe_branch_field = rel_get_field (dir_listing, x, 0);
+          maybe_branch = rel_field_str (maybe_branch_field);
 
           if (only_branch && str_cmp (only_branch, maybe_branch))
             continue;
@@ -389,9 +421,12 @@ arch_log_versions (t_uchar * tree_root,
               t_uchar * branch_path;
 
               branch_path = file_name_in_vicinity (0, cat_dir, maybe_branch);
-              rel_add_records (&branches, rel_make_record (maybe_branch, branch_path, 0), 0);
+              rel_field_ref (maybe_branch_field);
+              rel_add_records (&branches, rel_make_record_2_taking (maybe_branch_field, rel_make_field_str (branch_path)), 0);
               lim_free (0, branch_path);
             }
+
+          rel_field_unref (maybe_branch_field);
           lim_free (0, branch_cat);
         }
 
@@ -406,24 +441,26 @@ arch_log_versions (t_uchar * tree_root,
    */
 
 
-  versions = 0;
+  versions = rel_table_nil;
   b_lim = rel_n_records (branches);
   for (b = 0; b < b_lim; ++b)
     {
-      t_uchar * branch;
-      t_uchar * branch_dir;
+      const t_uchar * branch;
+      const t_uchar * branch_dir;
 
-      branch = branches[b][0];
-      branch_dir = branches[b][1];
+      branch = rel_peek_str (branches, b, 0);
+      branch_dir = rel_peek_str (branches, b, 1);
 
       dir_listing = directory_files (branch_dir);
       lim = rel_n_records (dir_listing);
       for (x = 0; x < lim; ++x)
         {
-          t_uchar * maybe_version;
-          t_uchar * version_pkg;
+          rel_field maybe_version_field;
+          const t_uchar * maybe_version;
+          t_uchar * version_pkg = 0;
 
-          maybe_version = dir_listing[x][0];
+          maybe_version_field = rel_get_field (dir_listing, x, 0);
+          maybe_version = rel_field_str (maybe_version_field);
 
           if (only_version && str_cmp (only_version, maybe_version))
             continue;
@@ -437,9 +474,13 @@ arch_log_versions (t_uchar * tree_root,
               t_uchar * version_path;
 
               version_path = file_name_in_vicinity (0, branch_dir, maybe_version);
-              rel_add_records (&versions, rel_make_record (maybe_version, version_path, 0), 0);
+              rel_field_ref (maybe_version_field);
+              rel_add_records (&versions, rel_make_record_2_taking (maybe_version_field, rel_make_field_str (version_path)), 0);
               lim_free (0, version_path);
             }
+
+
+          rel_field_unref (maybe_version_field);
           lim_free (0, version_pkg);
         }
 
@@ -453,25 +494,25 @@ arch_log_versions (t_uchar * tree_root,
    * For each version, get a list of each archive for it in the log
    */
 
-  log_versions = 0;
+  log_versions = rel_table_nil;
   v_lim = rel_n_records (versions);
   for (v = 0; v < v_lim; ++v)
     {
-      t_uchar * version;
-      t_uchar * vsn_dir;
+      const t_uchar * version;
+      const t_uchar * vsn_dir;
 
-      version = versions[v][0];
-      vsn_dir = versions[v][1];
+      version = rel_peek_str (versions, v, 0);
+      vsn_dir = rel_peek_str (versions, v, 1);
 
       dir_listing = directory_files (vsn_dir);
       lim = rel_n_records (dir_listing);
       for (x = 0; x < lim; ++x)
         {
-          t_uchar * maybe_archive;
-          t_uchar * archive_path;
-          t_uchar * patch_log_path;
+          const t_uchar * maybe_archive;
+          t_uchar * archive_path = 0;
+          t_uchar * patch_log_path = 0;
 
-          maybe_archive = dir_listing[x][0];
+          maybe_archive = rel_peek_str (dir_listing, x, 0);
 
           if (only_archive && str_cmp (only_archive, maybe_archive))
             continue;
@@ -487,7 +528,7 @@ arch_log_versions (t_uchar * tree_root,
               t_uchar * fqversion;
 
               fqversion = arch_fully_qualify (maybe_archive, version);
-              rel_add_records (&log_versions, rel_make_record (fqversion, patch_log_path, 0), 0);
+              rel_add_records (&log_versions, rel_make_record_2_taking (rel_make_field_str (fqversion), rel_make_field_str (patch_log_path)), 0);
               lim_free (0, fqversion);
             }
 
@@ -512,15 +553,17 @@ arch_log_versions (t_uchar * tree_root,
 
 
 t_uchar *
-arch_highest_patch_level (t_uchar * tree_root, t_uchar * archive, t_uchar * version)
+arch_highest_patch_level (const t_uchar * tree_root,
+                          const t_uchar * archive,
+                          const t_uchar * version)
 {
-  rel_table logs = 0;
+  rel_table logs = rel_table_nil;
   t_uchar * answer = 0;
 
   logs = arch_logs (tree_root, archive, version, 0);
 
-  if (logs)
-    answer = str_save (0, logs[rel_n_records(logs) - 1][0]);
+  if (rel_n_records (logs))
+    answer = str_save (0, rel_peek_str (logs, rel_n_records(logs) - 1, 0));
 
   rel_free_table (logs);
   return answer;
@@ -528,17 +571,19 @@ arch_highest_patch_level (t_uchar * tree_root, t_uchar * archive, t_uchar * vers
 
 
 t_uchar *
-arch_latest_logged_version (t_uchar * tree_root, t_uchar * archive, t_uchar * package)
+arch_latest_logged_version (const t_uchar * tree_root,
+                            const t_uchar * archive,
+                            const t_uchar * package)
 {
-  rel_table log_versions = 0;
+  rel_table log_versions = rel_table_nil;
   t_uchar * answer = 0;
 
   log_versions = arch_log_versions (tree_root, archive, 0, package, 0);
   arch_sort_table_by_name_field (1, log_versions, 0);
 
-  if (log_versions)
+  if (rel_n_records (log_versions))
     {
-      answer = arch_parse_package_name (arch_ret_non_archive, 0, log_versions[0][0]);
+      answer = arch_parse_package_name (arch_ret_non_archive, 0, rel_peek_str (log_versions, 0, 0));
     }
 
   rel_free_table (log_versions);
@@ -547,7 +592,10 @@ arch_latest_logged_version (t_uchar * tree_root, t_uchar * archive, t_uchar * pa
 
 
 rel_table
-arch_logs (t_uchar * tree_root, t_uchar * archive, t_uchar * version, int full)
+arch_logs (const t_uchar * tree_root,
+           const t_uchar * archive,
+           const t_uchar * version,
+           int full)
 {
   rel_table answer;             /* [0] == lvl, [2] == path */
   t_uchar * log_dir;
@@ -555,7 +603,7 @@ arch_logs (t_uchar * tree_root, t_uchar * archive, t_uchar * version, int full)
   int lim;
   int x;
 
-  answer = 0;
+  answer = rel_table_nil;
 
   log_dir = arch_log_dir (tree_root, archive, version);
 
@@ -567,23 +615,22 @@ arch_logs (t_uchar * tree_root, t_uchar * archive, t_uchar * version, int full)
   lim = rel_n_records (dir_listing);
   for (x = 0; x < lim; ++x)
     {
-      if (arch_valid_patch_level_name (dir_listing[x][0]))
+      if (arch_valid_patch_level_name (rel_peek_str (dir_listing, x, 0)))
         {
           t_uchar * path = 0;
 
-          path = file_name_in_vicinity (0, log_dir, dir_listing[x][0]);
+          path = file_name_in_vicinity (0, log_dir, rel_peek_str (dir_listing, x, 0));
 
           if (!full)
             {
-              rel_add_records (&answer, rel_make_record (dir_listing[x][0], path, 0), 0);
+              rel_add_records (&answer, rel_make_record_2_taking (rel_get_field (dir_listing, x, 0), rel_make_field_str (path)), 0);
             }
           else
             {
               t_uchar * full_name = 0;
 
-              full_name = str_alloc_cat_many (0, archive, "/", version, "--", dir_listing[x][0], str_end);
-              rel_add_records (&answer, rel_make_record (full_name, path, 0), 0);
-
+              full_name = str_alloc_cat_many (0, archive, "/", version, "--", rel_peek_str (dir_listing, x, 0), str_end);
+              rel_add_records (&answer, rel_make_record_2_taking (rel_make_field_str (full_name), rel_make_field_str (path)), 0);
               lim_free (0, full_name);
             }
 
@@ -601,11 +648,12 @@ arch_logs (t_uchar * tree_root, t_uchar * archive, t_uchar * version, int full)
   return answer;
 }
 
+
 int
-arch_valid_log_file (t_uchar * log)
+arch_valid_log_file (const t_uchar * log)
 {
   size_t len;
-  t_uchar * pos;
+  const t_uchar * pos;
 
   len = str_length (log);
   pos = log;
@@ -615,17 +663,17 @@ arch_valid_log_file (t_uchar * log)
 
   while (1)
     {
-      t_uchar * eol;
-      t_uchar * colon;
+      const t_uchar * eol;
+      const t_uchar * colon;
 
-      eol = str_chr_index (pos, '\n');
+      eol = (const t_uchar *)str_chr_index (pos, '\n');
       if (!eol)
         eol = pos + len;
 
       if (eol == pos)
         break;
 
-      colon = str_chr_index_n (pos, eol - pos, ':');
+      colon = (const t_uchar *)str_chr_index_n (pos, eol - pos, ':');
       if (!colon)
         return 0;
 
@@ -662,10 +710,13 @@ arch_valid_log_file (t_uchar * log)
 
 
 void
-arch_parse_log (rel_table * headers_list, assoc_table * headers, t_uchar ** body, t_uchar * log)
+arch_parse_log (rel_table * headers_list,
+                assoc_table * headers,
+                const t_uchar ** body,
+                const t_uchar * log)
 {
   size_t len;
-  t_uchar * pos;
+  const t_uchar * pos;
 
   len = str_length (log);
   pos = log;
@@ -675,19 +726,19 @@ arch_parse_log (rel_table * headers_list, assoc_table * headers, t_uchar ** body
 
   while (1)
     {
-      t_uchar * eol;
-      t_uchar * colon;
-      t_uchar * field_name;
-      t_uchar * field_value;
+      const t_uchar * eol;
+      const t_uchar * colon;
+      t_uchar * field_name = 0;
+      t_uchar * field_value = 0;
 
-      eol = str_chr_index (pos, '\n');
+      eol = (const t_uchar *)str_chr_index (pos, '\n');
       if (!eol)
         eol = pos + len;
 
       if (eol == pos)
         break;
 
-      colon = str_chr_index_n (pos, eol - pos, ':');
+      colon = (const t_uchar *)str_chr_index_n (pos, eol - pos, ':');
       if (!colon)
         panic ("bogusly formatted log message (missing field name in headers)");
 
@@ -721,9 +772,9 @@ arch_parse_log (rel_table * headers_list, assoc_table * headers, t_uchar ** body
       field_value = str_save_n (0, pos, eol - pos);
 
       if (headers)
-        assoc_set (headers, field_name, field_value);
+        assoc_set_taking (headers, rel_make_field_str (field_name), rel_make_field_str (field_value));
       if (headers_list)
-        rel_add_records (headers_list, rel_make_record (field_name, field_value, 0), 0);
+        rel_add_records (headers_list, rel_make_record_2_taking (rel_make_field_str (field_name), rel_make_field_str (field_value)), 0);
 
       lim_free (0, field_name);
       lim_free (0, field_value);
@@ -747,12 +798,15 @@ arch_parse_log (rel_table * headers_list, assoc_table * headers, t_uchar ** body
 
 
 void
-arch_print_headers_summary (int out_fd, int indent_level, assoc_table headers, int summarized_headers)
+arch_print_headers_summary (int out_fd,
+                            int indent_level,
+                            assoc_table headers,
+                            int summarized_headers)
 {
   if (arch_include_date & summarized_headers)
     {
-      t_uchar * d;
-      t_uchar * separator;
+      const t_uchar * d;
+      const t_uchar * separator;
 
       if (arch_include_creator & summarized_headers)
         {
@@ -763,7 +817,7 @@ arch_print_headers_summary (int out_fd, int indent_level, assoc_table headers, i
           separator = "\n";
         }
 
-      d = assoc_ref (headers, "standard-date");
+      d = assoc_get_str_taking (headers, rel_make_field_str ("standard-date"));
 
       if (!d)
         safe_printfmt (out_fd, "%*s<<<no date: header>>>%s", indent_level, "", separator);
@@ -773,7 +827,7 @@ arch_print_headers_summary (int out_fd, int indent_level, assoc_table headers, i
 
   if (arch_include_creator & summarized_headers)
     {
-      t_uchar * c;
+      const t_uchar * c;
       int creator_indent;
 
       if (arch_include_date & summarized_headers)
@@ -785,7 +839,7 @@ arch_print_headers_summary (int out_fd, int indent_level, assoc_table headers, i
           creator_indent = indent_level;
         }
 
-      c = assoc_ref (headers, "creator");
+      c = assoc_get_str_taking (headers, rel_make_field_str ("creator"));
       if (!c)
         safe_printfmt (out_fd, "%*s<<<no creator: header>>>\n", creator_indent, "");
       else
@@ -794,25 +848,60 @@ arch_print_headers_summary (int out_fd, int indent_level, assoc_table headers, i
 
   if (arch_include_summary & summarized_headers)
     {
-      t_uchar * s;
+      const t_uchar * s;
 
-      s = assoc_ref (headers, "summary");
+      s = assoc_get_str_taking (headers, rel_make_field_str ("summary"));
       if (!s)
         safe_printfmt (out_fd, "%*s<<<no summary: header>>>\n", indent_level, "");
       else
         {
-          rel_table s_lines = 0;
-          int x;
+          /* Iterate over the contents of the summary field.
+           *
+           * For each non-empty line, indent by `indent_level' spaces,
+           * skip the leading whitespace of the summary line,
+           * print all but the trailing whitespace on that line,
+           * and print a newline.
+           */
 
-          s_lines = rel_nl_split (s);
-
-          for (x = 0; x < rel_n_records (s_lines); ++x)
+          while (*s)
             {
-              s_lines[x][0] = trim_surrounding_ws (s_lines[x][0]);
-              safe_printfmt (out_fd, "%*s%s\n", indent_level, "", s_lines[x][0]);
-            }
+              const t_uchar * this_line_start;
+              const t_uchar * this_line_scan;
+              const t_uchar * this_line_end;
+              int this_line_len;
 
-          rel_free_table (s_lines);
+              this_line_start = s;
+
+              /* Skip leading whitespace.   Note that this will skip
+               * over the newlines of entirely blank lines because
+               * we are using `char_is_space'.  On the second and
+               * subsequent pass through this loop, it will also skip
+               * over the final newline of the previous line.
+               */
+              while (*this_line_start && char_is_space (*this_line_start))
+                ++this_line_start;
+
+              /* Find last non-whitespace character on this line.
+               * Leave `this_line_scan' at either the final 0 of the 
+               * string or at the newline that terminates this line.
+               */
+              this_line_scan = this_line_start;
+              this_line_end = this_line_start;
+              while (*this_line_scan && !('\n' == *this_line_scan))
+                {
+                  if (!char_is_blank (*this_line_scan))
+                    this_line_end = this_line_scan + 1;
+                  ++this_line_scan;
+                }
+
+              this_line_len = (int)(this_line_end - this_line_start);
+              invariant (this_line_len >= 0);
+              invariant ((size_t)this_line_len <= (this_line_end - this_line_start));
+
+              safe_printfmt (out_fd, "%*s%.*s\n", indent_level, "", this_line_len, this_line_start);
+
+              s = this_line_scan;
+            }
         }
     }
 
@@ -822,27 +911,27 @@ arch_print_headers_summary (int out_fd, int indent_level, assoc_table headers, i
       t_uchar * this_revision = 0;
       t_uchar * this_fqrevision = 0;
       t_uchar * raw_merges = 0;
-      rel_table new_patches = 0;
+      rel_table new_patches = rel_table_nil;
       int any = 0;
       int x;
 
-      this_archive = trim_surrounding_ws (str_save (0, assoc_ref (headers, "archive")));
-      this_revision = trim_surrounding_ws (str_save (0, assoc_ref (headers, "revision")));
+      this_archive = str_save_trimming (0, assoc_get_str_taking (headers, rel_make_field_str ("archive")));
+      this_revision = str_save_trimming (0, assoc_get_str_taking (headers, rel_make_field_str ("revision")));
       this_fqrevision = arch_fully_qualify (this_archive, this_revision);
 
-      raw_merges = trim_surrounding_ws (str_save (0, assoc_ref (headers, "new-patches")));
+      raw_merges = str_save_trimming (0, assoc_get_str_taking (headers, rel_make_field_str ("new-patches")));
 
       new_patches = rel_ws_split (raw_merges);
       arch_sort_table_by_name_field (0, new_patches, 0);
 
       for (x = 0; x < rel_n_records (new_patches); ++x)
         {
-          if (arch_valid_package_name (new_patches[x][0], arch_req_archive, arch_req_patch_level, 0)
-              && str_cmp (new_patches[x][0], this_fqrevision))
+          if (arch_valid_package_name (rel_peek_str (new_patches, x, 0), arch_req_archive, arch_req_patch_level, 0)
+              && str_cmp (rel_peek_str (new_patches, x, 0), this_fqrevision))
             {
               t_uchar * patch_archive = 0;
 
-              patch_archive = arch_parse_package_name (arch_ret_archive, 0, new_patches[x][0]);
+              patch_archive = arch_parse_package_name (arch_ret_archive, 0, rel_peek_str (new_patches, x, 0));
 
               if (str_cmp (patch_archive, this_archive)
                   ? (summarized_headers & arch_include_foreign_merges)
@@ -856,7 +945,7 @@ arch_print_headers_summary (int out_fd, int indent_level, assoc_table headers, i
                       safe_printfmt (out_fd, "%*smerges in:\n", indent_level, "");
                       any = 1;
                     }
-                  safe_printfmt (out_fd, "%*s%s\n", indent_level + 2, "", new_patches[x][0]);
+                  safe_printfmt (out_fd, "%*s%s\n", indent_level + 2, "", rel_peek_str (new_patches, x, 0));
                 }
 
               lim_free (0, patch_archive);
@@ -872,8 +961,8 @@ arch_print_headers_summary (int out_fd, int indent_level, assoc_table headers, i
 }
 
 
-static char *
-no_dot (char *name)
+static const char *
+no_dot (const char *name)
 {
   if (name[0] == '.' && name[1] == '/')
     return name + 2;
@@ -881,8 +970,12 @@ no_dot (char *name)
     return name;
 }
 
+
 void
-arch_print_log_list_header (int out_fd, t_uchar * header, rel_table list, int field)
+arch_print_log_list_header (int out_fd,
+                            const t_uchar * header,
+                            rel_table list,
+                            int field)
 {
   int num_recs = rel_n_records (list);
   int escape_classes = arch_escape_classes;
@@ -902,7 +995,7 @@ arch_print_log_list_header (int out_fd, t_uchar * header, rel_table list, int fi
           t_uchar * escape_tmp = 0;
           size_t len_this;
 
-          escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, no_dot (list[x][field]));
+          escape_tmp = pika_save_escape_iso8859_1 (0, 0, escape_classes, no_dot (rel_peek_str (list, x, field)));
           len_this = str_length (escape_tmp);
 
           if (!any_on_this_line)
@@ -935,7 +1028,11 @@ arch_print_log_list_header (int out_fd, t_uchar * header, rel_table list, int fi
 
 
 void
-arch_print_log_pairs_header (int out_fd, t_uchar * header, rel_table list, int field_a, int field_b)
+arch_print_log_pairs_header (int out_fd,
+                             const t_uchar * header,
+                             rel_table list,
+                             int field_a,
+                             int field_b)
 {
   int num_recs = rel_n_records (list);
   int escape_classes = arch_escape_classes;
@@ -953,8 +1050,8 @@ arch_print_log_pairs_header (int out_fd, t_uchar * header, rel_table list, int f
         {
           t_uchar * escape_tmp_a;
           t_uchar * escape_tmp_b;
-          escape_tmp_a = pika_save_escape_iso8859_1 (0, 0, escape_classes, list[x][field_a]);
-          escape_tmp_b = pika_save_escape_iso8859_1 (0, 0, escape_classes, list[x][field_b]);
+          escape_tmp_a = pika_save_escape_iso8859_1 (0, 0, escape_classes, rel_peek_str (list, x, field_a));
+          escape_tmp_b = pika_save_escape_iso8859_1 (0, 0, escape_classes, rel_peek_str (list, x, field_b));
           safe_printfmt (out_fd, "%s%s %s", (once ? "\n    " : ""),
                          no_dot (escape_tmp_a), no_dot (escape_tmp_b));
           once = 1;
@@ -967,7 +1064,10 @@ arch_print_log_pairs_header (int out_fd, t_uchar * header, rel_table list, int f
 
 
 void
-arch_copy_to_patch_log (t_uchar * tree_root, t_uchar * archive, t_uchar * revision, t_uchar * log_file)
+arch_copy_to_patch_log (const t_uchar * tree_root,
+                        const t_uchar * archive,
+                        const t_uchar * revision,
+                        const t_uchar * log_file)
 {
   int ign;
   t_uchar * version = 0;
@@ -1001,8 +1101,12 @@ arch_copy_to_patch_log (t_uchar * tree_root, t_uchar * archive, t_uchar * revisi
   lim_free (0, tmp_path);
 }
 
+
 void
-arch_rename_to_patch_log (t_uchar * tree_root, t_uchar * archive, t_uchar * revision, t_uchar * log_file)
+arch_rename_to_patch_log (const t_uchar * tree_root,
+                          const t_uchar * archive,
+                          const t_uchar * revision,
+                          const t_uchar * log_file)
 {
   t_uchar * version = 0;
   t_uchar * lvl = 0;
@@ -1024,11 +1128,12 @@ arch_rename_to_patch_log (t_uchar * tree_root, t_uchar * archive, t_uchar * revi
   lim_free (0, dest_path);
 }
 
+
 rel_table
-arch_all_logs (t_uchar * tree_root)
+arch_all_logs (const t_uchar * tree_root)
 {
-  rel_table log_versions = 0;
-  rel_table answer = 0;
+  rel_table log_versions = rel_table_nil;
+  rel_table answer = rel_table_nil;
   int x;
 
   log_versions = arch_log_versions (tree_root, 0, 0, 0, 0);
@@ -1037,12 +1142,12 @@ arch_all_logs (t_uchar * tree_root)
     {
       t_uchar * archive = 0;
       t_uchar * version = 0;
-      rel_table in_this_log = 0;
+      rel_table in_this_log = rel_table_nil;
 
       archive = arch_parse_package_name (arch_ret_archive, 0,
-                                         log_versions[x][0]);
+                                         rel_peek_str (log_versions, x, 0));
       version = arch_parse_package_name (arch_ret_non_archive, 0,
-                                         log_versions[x][0]);
+                                         rel_peek_str (log_versions, x, 0));
 
       in_this_log = arch_logs (tree_root, archive, version, 1);
       rel_append_x (&answer, in_this_log);
