@@ -27,9 +27,11 @@
 
 
 void
-arch_make_pristine (t_uchar * tree_root, t_uchar * archive, t_uchar * revision)
+arch_make_pristine (const t_uchar * tree_root,
+                    const t_uchar * archive,
+                    const t_uchar * revision)
 {
-  rel_table inventory = 0;
+  rel_table inventory = rel_table_nil;
   t_uchar * tmp_pristine = 0;
 
   inventory = arch_source_inventory (tree_root, 1, 0, 0);
@@ -46,7 +48,11 @@ arch_make_pristine (t_uchar * tree_root, t_uchar * archive, t_uchar * revision)
 
 
 void
-arch_add_pristine (int chatter_fd, t_uchar * tree_root, struct arch_archive * arch, t_uchar * archive, t_uchar * revision)
+arch_add_pristine (int chatter_fd,
+                   const t_uchar * tree_root,
+                   struct arch_archive * arch,
+                   const t_uchar * archive,
+                   const t_uchar * revision)
 {
   t_uchar * tmp_path = 0;
   t_uchar * cache_dir = 0;
@@ -64,7 +70,9 @@ arch_add_pristine (int chatter_fd, t_uchar * tree_root, struct arch_archive * ar
 
 
 t_uchar *
-arch_pristine_loc (t_uchar * archive, t_uchar * revision, int locked_p)
+arch_pristine_loc (const t_uchar * archive,
+                   const t_uchar * revision,
+                   int locked_p)
 {
   t_uchar * category = 0;
   t_uchar * branch = 0;
@@ -89,7 +97,10 @@ arch_pristine_loc (t_uchar * archive, t_uchar * revision, int locked_p)
 
 
 t_uchar *
-arch_pristine_path (t_uchar * tree_root, t_uchar * archive, t_uchar * revision, int locked_p)
+arch_pristine_path (const t_uchar * tree_root,
+                    const t_uchar * archive,
+                    const t_uchar * revision,
+                    int locked_p)
 {
   t_uchar * loc = 0;
   t_uchar * answer = 0;
@@ -101,8 +112,12 @@ arch_pristine_path (t_uchar * tree_root, t_uchar * archive, t_uchar * revision,
   return answer;
 }
 
+
 void
-arch_install_pristine (t_uchar * tree_root, t_uchar * archive, t_uchar * revision, t_uchar * source)
+arch_install_pristine (const t_uchar * tree_root,
+                       const t_uchar * archive,
+                       const t_uchar * revision,
+                       const t_uchar * source)
 {
   t_uchar * loc = 0;
   t_uchar * path = 0;
@@ -122,7 +137,9 @@ arch_install_pristine (t_uchar * tree_root, t_uchar * archive, t_uchar * revisio
 
 
 void
-arch_lock_pristine (t_uchar * tree_root, t_uchar * archive, t_uchar * revision)
+arch_lock_pristine (const t_uchar * tree_root,
+                    const t_uchar * archive,
+                    const t_uchar * revision)
 {
   t_uchar * locked_path = 0;
   t_uchar * unlocked_path = 0;
@@ -151,8 +168,11 @@ arch_lock_pristine (t_uchar * tree_root, t_uchar * archive, t_uchar * revision)
   lim_free (0, unlocked_path);
 }
 
+
 void
-arch_unlock_pristine (t_uchar * tree_root, t_uchar * archive, t_uchar * revision)
+arch_unlock_pristine (const t_uchar * tree_root,
+                      const t_uchar * archive,
+                      const t_uchar * revision)
 {
   t_uchar * locked_path = 0;
   t_uchar * unlocked_path = 0;
@@ -182,11 +202,10 @@ arch_unlock_pristine (t_uchar * tree_root, t_uchar * archive, t_uchar * revision
 }
 
 
-
 rel_table
-arch_pristines (t_uchar * tree_root,
-                t_uchar * archive_limit,
-                t_uchar * limit,
+arch_pristines (const t_uchar * tree_root,
+                const t_uchar * archive_limit,
+                const t_uchar * limit,
                 int pristine_types)
 {
   int here_fd;
@@ -195,10 +214,10 @@ arch_pristines (t_uchar * tree_root,
   t_uchar * category = 0;
   t_uchar * branch = 0;
   t_uchar * version = 0;
-  t_uchar * revision = 0;
+  const t_uchar * revision;
   t_uchar * locked_in_this_tree_path = 0;
   t_uchar * unlocked_in_this_tree_path = 0;
-  rel_table answer = 0;
+  rel_table answer = rel_table_nil;
   int x;
 
   here_fd = safe_open (".", O_RDONLY, 0);
@@ -256,12 +275,12 @@ arch_pristines (t_uchar * tree_root,
             }
 
           {
-            rel_table maybe_categories = 0;
+            rel_table maybe_categories = rel_table_nil;
             int c;
 
             if (category)
               {
-                rel_add_records (&maybe_categories, rel_make_record (category, 0), 0);
+                rel_add_records (&maybe_categories, rel_singleton_record_taking (rel_make_field_str (category)), 0);
               }
             else
               {
@@ -272,17 +291,17 @@ arch_pristines (t_uchar * tree_root,
             for (c = 0; c < rel_n_records (maybe_categories); ++c)
               {
 
-                if (arch_valid_package_name (maybe_categories[c][0], arch_no_archive, arch_req_category, 0)
-                    && (!vu_lstat (&errn, maybe_categories[c][0], &statb) && S_ISDIR (statb.st_mode)))
+                if (arch_valid_package_name (rel_peek_str (maybe_categories, c, 0), arch_no_archive, arch_req_category, 0)
+                    && (!vu_lstat (&errn, rel_peek_str (maybe_categories, c, 0), &statb) && S_ISDIR (statb.st_mode)))
                   {
-                    rel_table maybe_branches = 0;
+                    rel_table maybe_branches = rel_table_nil;
                     int b;
 
-                    safe_chdir (maybe_categories[c][0]);
+                    safe_chdir (rel_peek_str (maybe_categories, c, 0));
 
                     if (branch)
                       {
-                        rel_add_records (&maybe_branches, rel_make_record (branch, 0), 0);
+                        rel_add_records (&maybe_branches, rel_singleton_record_taking (rel_make_field_str (branch)), 0);
                       }
                     else
                       {
@@ -292,17 +311,17 @@ arch_pristines (t_uchar * tree_root,
                     for (b = 0; b < rel_n_records (maybe_branches); ++b)
                       {
 
-                        if (arch_valid_package_name (maybe_branches[b][0], arch_no_archive, arch_req_package, 0)
-                            && (!vu_lstat (&errn, maybe_branches[b][0], &statb) && S_ISDIR (statb.st_mode)))
+                        if (arch_valid_package_name (rel_peek_str (maybe_branches, b, 0), arch_no_archive, arch_req_package, 0)
+                            && (!vu_lstat (&errn, rel_peek_str (maybe_branches, b, 0), &statb) && S_ISDIR (statb.st_mode)))
                           {
-                            rel_table maybe_versions = 0;
+                            rel_table maybe_versions = rel_table_nil;
                             int v;
 
-                            safe_chdir (maybe_branches[b][0]);
+                            safe_chdir (rel_peek_str (maybe_branches, b, 0));
 
                             if (version)
                               {
-                                rel_add_records (&maybe_versions, rel_make_record (version, 0), 0);
+                                rel_add_records (&maybe_versions, rel_singleton_record_taking (rel_make_field_str (version)), 0);
                               }
                             else
                               {
@@ -311,17 +330,17 @@ arch_pristines (t_uchar * tree_root,
 
                             for (v = 0; v < rel_n_records (maybe_versions); ++v)
                               {
-                                if (arch_valid_package_name (maybe_versions[v][0], arch_no_archive, arch_req_version, 0)
-                                    && (!vu_lstat (&errn, maybe_versions[v][0], &statb) && S_ISDIR (statb.st_mode)))
+                                if (arch_valid_package_name (rel_peek_str (maybe_versions, v, 0), arch_no_archive, arch_req_version, 0)
+                                    && (!vu_lstat (&errn, rel_peek_str (maybe_versions, v, 0), &statb) && S_ISDIR (statb.st_mode)))
                                   {
-                                    rel_table maybe_archives = 0;
+                                    rel_table maybe_archives = rel_table_nil;
                                     int a;
 
-                                    safe_chdir (maybe_versions[v][0]);
+                                    safe_chdir (rel_peek_str (maybe_versions, v, 0));
 
                                     if (archive_limit)
                                       {
-                                        rel_add_records (&maybe_archives, rel_make_record (archive_limit, 0), 0);
+                                        rel_add_records (&maybe_archives, rel_singleton_record_taking (rel_make_field_str (archive_limit)), 0);
                                       }
                                     else
                                       {
@@ -330,16 +349,16 @@ arch_pristines (t_uchar * tree_root,
 
                                     for (a = 0; a < rel_n_records (maybe_archives); ++a)
                                       {
-                                        if (arch_valid_archive_name (maybe_archives[a][0]) && (!vu_lstat (&errn, maybe_archives[a][0], &statb) && S_ISDIR (statb.st_mode)))
+                                        if (arch_valid_archive_name (rel_peek_str (maybe_archives, a, 0)) && (!vu_lstat (&errn, rel_peek_str (maybe_archives, a, 0), &statb) && S_ISDIR (statb.st_mode)))
                                           {
-                                            rel_table maybe_revisions = 0;
+                                            rel_table maybe_revisions = rel_table_nil;
                                             int r;
 
-                                            safe_chdir (maybe_archives[a][0]);
+                                            safe_chdir (rel_peek_str (maybe_archives, a, 0));
 
                                             if (revision)
                                               {
-                                                rel_add_records (&maybe_revisions, rel_make_record (revision, 0), 0);
+                                                rel_add_records (&maybe_revisions, rel_singleton_record_taking (rel_make_field_str (revision)), 0);
                                               }
                                             else
                                               {
@@ -348,13 +367,13 @@ arch_pristines (t_uchar * tree_root,
 
                                             for (r = 0; r < rel_n_records (maybe_revisions); ++r)
                                               {
-                                                if (arch_valid_package_name (maybe_revisions[r][0], arch_no_archive, arch_req_patch_level, 0)
-                                                    && (!vu_lstat (&errn, maybe_revisions[r][0], &statb) && S_ISDIR (statb.st_mode)))
+                                                if (arch_valid_package_name (rel_peek_str (maybe_revisions, r, 0), arch_no_archive, arch_req_patch_level, 0)
+                                                    && (!vu_lstat (&errn, rel_peek_str (maybe_revisions, r, 0), &statb) && S_ISDIR (statb.st_mode)))
                                                   {
                                                     t_uchar * fqr = 0;
 
-                                                    fqr = arch_fully_qualify (maybe_archives[a][0], maybe_revisions[r][0]);
-                                                    rel_add_records (&answer, rel_make_record (fqr, 0), 0);
+                                                    fqr = arch_fully_qualify (rel_peek_str (maybe_archives, a, 0), rel_peek_str (maybe_revisions, r, 0));
+                                                    rel_add_records (&answer, rel_singleton_record_taking (rel_make_field_str (fqr)), 0);
                                                     lim_free (0, fqr);
                                                   }
                                               }
@@ -392,7 +411,6 @@ arch_pristines (t_uchar * tree_root,
   lim_free (0, category);
   lim_free (0, branch);
   lim_free (0, version);
-  lim_free (0, revision);
   lim_free (0, locked_in_this_tree_path);
   lim_free (0, unlocked_in_this_tree_path);
 
@@ -402,9 +420,9 @@ arch_pristines (t_uchar * tree_root,
 
 t_uchar *
 arch_find_pristine (int * is_locked,
-                    t_uchar * tree_root_or_root_dir,
-                    t_uchar * archive,
-                    t_uchar * revision,
+                    const t_uchar * tree_root_or_root_dir,
+                    const t_uchar * archive,
+                    const t_uchar * revision,
                     int pristine_types,
                     enum arch_pristine_search_scope scope)
 {
@@ -442,8 +460,8 @@ arch_find_pristine (int * is_locked,
   else
     {
       t_uchar * tree_root_dir = 0;
-      rel_table sibling_files = 0;
-      rel_table sibling_paths = 0;
+      rel_table sibling_files = rel_table_nil;
+      rel_table sibling_paths = rel_table_nil;
       int x;
 
       if (scope == arch_cache_dir_pristine_search)
@@ -460,17 +478,17 @@ arch_find_pristine (int * is_locked,
         {
           for (x = 0; x < rel_n_records (sibling_files); ++x)
             {
-              if (char_is_alnum (sibling_files[x][0][0]))
+              if (char_is_alnum (rel_peek_str (sibling_files, x, 0)[0]))
                 {
                   int ign;
                   struct stat stat_buf;
                   t_uchar * path = 0;
 
-                  path = file_name_in_vicinity (0, tree_root_dir, sibling_files[x][0]);
+                  path = file_name_in_vicinity (0, tree_root_dir, rel_peek_str (sibling_files, x, 0));
 
                   if (!vu_lstat (&ign, path, &stat_buf) && S_ISDIR (stat_buf.st_mode))
                     {
-                      rel_add_records (&sibling_paths, rel_make_record (path, 0), 0);
+                      rel_add_records (&sibling_paths, rel_singleton_record_taking (rel_make_field_str (path)), 0);
                     }
                   lim_free (0, path);
                 }
@@ -483,7 +501,7 @@ arch_find_pristine (int * is_locked,
                   t_uchar * unlocked_sibling_path = 0;
                   int errno = 0;
 
-                  unlocked_sibling_path = file_name_in_vicinity (0, sibling_paths[x][0], unlocked_relpath);
+                  unlocked_sibling_path = file_name_in_vicinity (0, rel_peek_str (sibling_paths, x, 0), unlocked_relpath);
 
                   if (!vu_access (&errno, unlocked_sibling_path, F_OK))
                     {
@@ -513,7 +531,7 @@ arch_find_pristine (int * is_locked,
                   t_uchar * locked_sibling_path = 0;
                   int errno = 0;
 
-                  locked_sibling_path = file_name_in_vicinity (0, sibling_paths[x][0], locked_relpath);
+                  locked_sibling_path = file_name_in_vicinity (0, rel_peek_str (sibling_paths, x, 0), locked_relpath);
 
                   if (!vu_access (&errno, locked_sibling_path, F_OK))
                     {
